@@ -37,10 +37,13 @@ where
     type IntoIter = Box<dyn PolarsIterator<Item = Self::Item> + 'a>;
     fn into_iter(self) -> Self::IntoIter {
         Box::new(
-            self.downcast_iter()
-                .flatten()
-                .map(|x| x.copied())
-                .trust_my_length(self.len()),
+            // we know that we only iterate over length == self.len()
+            unsafe {
+                self.downcast_iter()
+                    .flatten()
+                    .map(|x| x.copied())
+                    .trust_my_length(self.len())
+            },
         )
     }
 }
@@ -59,7 +62,8 @@ impl<'a> IntoIterator for &'a BooleanChunked {
     type Item = Option<bool>;
     type IntoIter = Box<dyn PolarsIterator<Item = Self::Item> + 'a>;
     fn into_iter(self) -> Self::IntoIter {
-        Box::new(self.downcast_iter().flatten().trust_my_length(self.len()))
+        // we know that we only iterate over length == self.len()
+        unsafe { Box::new(self.downcast_iter().flatten().trust_my_length(self.len())) }
     }
 }
 
@@ -127,10 +131,12 @@ impl BooleanChunked {
            + ExactSizeIterator
            + DoubleEndedIterator
            + TrustedLen {
-        self.downcast_iter()
-            .map(BoolIterNoNull::new)
-            .flatten()
-            .trust_my_length(self.len())
+        // we know that we only iterate over length == self.len()
+        unsafe {
+            self.downcast_iter()
+                .flat_map(BoolIterNoNull::new)
+                .trust_my_length(self.len())
+        }
     }
 }
 
@@ -138,7 +144,8 @@ impl<'a> IntoIterator for &'a Utf8Chunked {
     type Item = Option<&'a str>;
     type IntoIter = Box<dyn PolarsIterator<Item = Self::Item> + 'a>;
     fn into_iter(self) -> Self::IntoIter {
-        Box::new(self.downcast_iter().flatten().trust_my_length(self.len()))
+        // we know that we only iterate over length == self.len()
+        unsafe { Box::new(self.downcast_iter().flatten().trust_my_length(self.len())) }
     }
 }
 
@@ -205,10 +212,12 @@ impl Utf8Chunked {
            + ExactSizeIterator
            + DoubleEndedIterator
            + TrustedLen {
-        self.downcast_iter()
-            .map(Utf8IterNoNull::new)
-            .flatten()
-            .trust_my_length(self.len())
+        // we know that we only iterate over length == self.len()
+        unsafe {
+            self.downcast_iter()
+                .flat_map(Utf8IterNoNull::new)
+                .trust_my_length(self.len())
+        }
     }
 }
 
@@ -216,13 +225,21 @@ impl<'a> IntoIterator for &'a ListChunked {
     type Item = Option<Series>;
     type IntoIter = Box<dyn PolarsIterator<Item = Self::Item> + 'a>;
     fn into_iter(self) -> Self::IntoIter {
-        Box::new(
-            self.downcast_iter()
-                .map(|arr| arr.iter())
-                .flatten()
-                .trust_my_length(self.len())
-                .map(|arr| arr.map(|arr| Series::try_from(("", arr)).unwrap())),
-        )
+        let dtype = self.inner_dtype().to_arrow();
+
+        // we know that we only iterate over length == self.len()
+        unsafe {
+            Box::new(
+                self.downcast_iter()
+                    .flat_map(|arr| arr.iter())
+                    .trust_my_length(self.len())
+                    .map(move |arr| {
+                        arr.map(|arr| {
+                            Series::try_from_unchecked("", vec![Arc::from(arr)], &dtype).unwrap()
+                        })
+                    }),
+            )
+        }
     }
 }
 
@@ -291,10 +308,12 @@ impl ListChunked {
            + ExactSizeIterator
            + DoubleEndedIterator
            + TrustedLen {
-        self.downcast_iter()
-            .map(ListIterNoNull::new)
-            .flatten()
-            .trust_my_length(self.len())
+        // we know that we only iterate over length == self.len()
+        unsafe {
+            self.downcast_iter()
+                .flat_map(ListIterNoNull::new)
+                .trust_my_length(self.len())
+        }
     }
 }
 
@@ -306,7 +325,8 @@ where
     type Item = Option<&'a T>;
     type IntoIter = Box<dyn PolarsIterator<Item = Self::Item> + 'a>;
     fn into_iter(self) -> Self::IntoIter {
-        Box::new(self.downcast_iter().flatten().trust_my_length(self.len()))
+        // we know that we only iterate over length == self.len()
+        unsafe { Box::new(self.downcast_iter().flatten().trust_my_length(self.len())) }
     }
 }
 
@@ -317,10 +337,12 @@ impl<T: PolarsObject> ObjectChunked<T> {
         &self,
     ) -> impl Iterator<Item = &T> + '_ + Send + Sync + ExactSizeIterator + DoubleEndedIterator + TrustedLen
     {
-        self.downcast_iter()
-            .map(|arr| arr.values().iter())
-            .flatten()
-            .trust_my_length(self.len())
+        // we know that we only iterate over length == self.len()
+        unsafe {
+            self.downcast_iter()
+                .flat_map(|arr| arr.values().iter())
+                .trust_my_length(self.len())
+        }
     }
 }
 
