@@ -966,32 +966,17 @@ mod test {
     // Single Chunk Null Check Parallel Iterator Tests.
     impl_par_iter_return_option_map_test!(
         uint32_par_iter_single_chunk_null_check_return_option_map,
-        {
-            UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            )
-        }
+        { UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),) }
     );
 
     impl_par_iter_return_option_filter_test!(
         uint32_par_iter_single_chunk_null_check_return_option_filter,
-        {
-            UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            )
-        }
+        { UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),) }
     );
 
     impl_par_iter_return_option_fold_test!(
         uint32_par_iter_single_chunk_null_check_return_option_fold,
-        {
-            UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            )
-        }
+        { UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),) }
     );
 
     // Many Chunk Parallel Iterator Tests.
@@ -1026,14 +1011,9 @@ mod test {
     impl_par_iter_return_option_map_test!(
         uint32_par_iter_many_chunk_null_check_return_option_map,
         {
-            let mut a = UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            );
-            let a_b = UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            );
+            let mut a =
+                UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE));
+            let a_b = UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE));
             a.append(&a_b);
             a
         }
@@ -1042,14 +1022,9 @@ mod test {
     impl_par_iter_return_option_filter_test!(
         uint32_par_iter_many_chunk_null_check_return_option_filter,
         {
-            let mut a = UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            );
-            let a_b = UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            );
+            let mut a =
+                UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE));
+            let a_b = UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE));
             a.append(&a_b);
             a
         }
@@ -1058,14 +1033,9 @@ mod test {
     impl_par_iter_return_option_fold_test!(
         uint32_par_iter_many_chunk_null_check_return_option_fold,
         {
-            let mut a = UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            );
-            let a_b = UInt32Chunked::new_from_opt_slice(
-                "a",
-                &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE),
-            );
+            let mut a =
+                UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE));
+            let a_b = UInt32Chunked::new("a", &generate_opt_uint32_vec(UINT32_CHUNKED_ARRAY_SIZE));
             a.append(&a_b);
             a
         }
