@@ -29,6 +29,7 @@ mod inner_mod {
     use arrow::bitmap::MutableBitmap;
     use num::{Float, Zero};
     use polars_arrow::bit_util::unset_bit_raw;
+    use polars_arrow::prelude::QuantileInterpolOptions;
     use polars_arrow::{kernels::rolling, trusted_len::PushUnchecked};
     use std::convert::TryFrom;
 
@@ -113,6 +114,89 @@ mod inner_mod {
             Series::try_from((self.name(), arr))
         }
 
+        /// Apply a rolling median (moving median) over the values in this array.
+        /// A window of length `window_size` will traverse the array. The values that fill this window
+        /// will (optionally) be weighted according to the `weights` vector.
+        pub fn rolling_median(&self, options: RollingOptions) -> Result<Series> {
+            match self.dtype() {
+                DataType::Float32 | DataType::Float64 => {
+                    check_input(options.window_size, options.min_periods)?;
+                    let ca = self.rechunk();
+
+                    let arr = ca.downcast_iter().next().unwrap();
+                    let arr = match self.has_validity() {
+                        false => rolling::no_nulls::rolling_quantile(
+                            arr.values(),
+                            0.5,
+                            QuantileInterpolOptions::Linear,
+                            options.window_size,
+                            options.min_periods,
+                            options.center,
+                            options.weights.as_deref(),
+                        ),
+                        _ => rolling::nulls::rolling_quantile(
+                            arr,
+                            0.5,
+                            QuantileInterpolOptions::Linear,
+                            options.window_size,
+                            options.min_periods,
+                            options.center,
+                            options.weights.as_deref(),
+                        ),
+                    };
+                    Series::try_from((self.name(), arr))
+                }
+                _ => {
+                    let s = self.cast(&DataType::Float64)?;
+                    s.rolling_median(options)
+                }
+            }
+        }
+
+        /// Apply a rolling quantile (moving quantile) over the values in this array.
+        /// A window of length `window_size` will traverse the array. The values that fill this window
+        /// will (optionally) be weighted according to the `weights` vector.
+        pub fn rolling_quantile(
+            &self,
+            quantile: f64,
+            interpolation: QuantileInterpolOptions,
+            options: RollingOptions,
+        ) -> Result<Series> {
+            match self.dtype() {
+                DataType::Float32 | DataType::Float64 => {
+                    check_input(options.window_size, options.min_periods)?;
+                    let ca = self.rechunk();
+
+                    let arr = ca.downcast_iter().next().unwrap();
+                    let arr = match self.has_validity() {
+                        false => rolling::no_nulls::rolling_quantile(
+                            arr.values(),
+                            quantile,
+                            interpolation,
+                            options.window_size,
+                            options.min_periods,
+                            options.center,
+                            options.weights.as_deref(),
+                        ),
+                        _ => rolling::nulls::rolling_quantile(
+                            arr,
+                            quantile,
+                            interpolation,
+                            options.window_size,
+                            options.min_periods,
+                            options.center,
+                            options.weights.as_deref(),
+                        ),
+                    };
+                    Series::try_from((self.name(), arr))
+                }
+                _ => {
+                    let s = self.cast(&DataType::Float64)?;
+                    s.rolling_quantile(quantile, interpolation, options)
+                }
+            }
+        }
+
         /// Apply a rolling min (moving min) over the values in this array.
         /// A window of length `window_size` will traverse the array. The values that fill this window
         /// will (optionally) be multiplied with the weights given by the `weights` vector. The resulting
@@ -193,45 +277,125 @@ mod inner_mod {
         }
     }
 
+    /// utility
+    fn window_edges(idx: usize, len: usize, window_size: usize, center: bool) -> (usize, usize) {
+        let (start, end) = if center {
+            (
+                idx.saturating_sub(window_size / 2),
+                std::cmp::min(len, idx + window_size / 2),
+            )
+        } else {
+            (idx.saturating_sub(window_size - 1), idx + 1)
+        };
+
+        (start, end - start)
+    }
+
     impl<T> ChunkRollApply for ChunkedArray<T>
     where
         T: PolarsNumericType,
         Self: IntoSeries,
     {
         /// Apply a rolling custom function. This is pretty slow because of dynamic dispatch.
-        fn rolling_apply(&self, window_size: usize, f: &dyn Fn(&Series) -> Series) -> Result<Self> {
-            if window_size >= self.len() {
-                return Ok(Self::full_null(self.name(), self.len()));
-            }
-            let ca = self.rechunk();
-            let arr = ca.downcast_iter().next().unwrap();
+        fn rolling_apply(
+            &self,
+            f: &dyn Fn(&Series) -> Series,
+            options: RollingOptions,
+        ) -> Result<Series> {
+            check_input(options.window_size, options.min_periods)?;
 
+            let ca = self.rechunk();
+            if options.weights.is_some()
+                && !matches!(self.dtype(), DataType::Float64 | DataType::Float32)
+            {
+                let s = self.cast(&DataType::Float64)?;
+                return s.rolling_apply(f, options);
+            }
+
+            if options.window_size >= self.len() {
+                return Ok(Self::full_null(self.name(), self.len()).into_series());
+            }
+
+            let len = self.len();
+            let arr = ca.downcast_iter().next().unwrap();
             let series_container =
                 ChunkedArray::<T>::new_from_slice("", &[T::Native::zero()]).into_series();
             let array_ptr = &series_container.chunks()[0];
             let ptr = Arc::as_ptr(array_ptr) as *mut dyn Array as *mut PrimitiveArray<T::Native>;
             let mut builder = PrimitiveChunkedBuilder::<T>::new(self.name(), self.len());
-            for _ in 0..window_size - 1 {
-                builder.append_null();
-            }
 
-            for offset in 0..self.len() + 1 - window_size {
-                let arr_window = arr.slice(offset, window_size);
+            if let Some(weights) = options.weights {
+                let weights_series = Float64Chunked::new("weights", &weights).into_series();
 
-                // Safety.
-                // ptr is not dropped as we are in scope
-                // We are also the only owner of the contents of the Arc
-                // we do this to reduce heap allocs.
-                unsafe {
-                    *ptr = arr_window;
+                let weights_series = weights_series.cast(self.dtype()).unwrap();
+
+                for idx in 0..len {
+                    let (start, size) = window_edges(idx, len, options.window_size, options.center);
+
+                    if size < options.min_periods {
+                        builder.append_null();
+                    } else {
+                        let arr_window = arr.slice(start, size);
+
+                        // Safety.
+                        // ptr is not dropped as we are in scope
+                        // We are also the only owner of the contents of the Arc
+                        // we do this to reduce heap allocs.
+                        unsafe {
+                            *ptr = arr_window;
+                        }
+
+                        let s = if size == options.window_size {
+                            f(&series_container.multiply(&weights_series).unwrap())
+                        } else {
+                            let weights_cutoff: Series = match self.dtype() {
+                                DataType::Float64 => weights_series
+                                    .f64()
+                                    .unwrap()
+                                    .into_iter()
+                                    .take(series_container.len())
+                                    .collect(),
+                                _ => weights_series // Float32 case
+                                    .f32()
+                                    .unwrap()
+                                    .into_iter()
+                                    .take(series_container.len())
+                                    .collect(),
+                            };
+                            f(&series_container.multiply(&weights_cutoff).unwrap())
+                        };
+
+                        let out = self.unpack_series_matching_type(&s)?;
+                        builder.append_option(out.get(0));
+                    }
                 }
 
-                let s = f(&series_container);
-                let out = self.unpack_series_matching_type(&s)?;
-                builder.append_option(out.get(0));
-            }
+                Ok(builder.finish().into_series())
+            } else {
+                for idx in 0..len {
+                    let (start, size) = window_edges(idx, len, options.window_size, options.center);
 
-            Ok(builder.finish())
+                    if size < options.min_periods {
+                        builder.append_null();
+                    } else {
+                        let arr_window = arr.slice(start, size);
+
+                        // Safety.
+                        // ptr is not dropped as we are in scope
+                        // We are also the only owner of the contents of the Arc
+                        // we do this to reduce heap allocs.
+                        unsafe {
+                            *ptr = arr_window;
+                        }
+
+                        let s = f(&series_container);
+                        let out = self.unpack_series_matching_type(&s)?;
+                        builder.append_option(out.get(0));
+                    }
+                }
+
+                Ok(builder.finish().into_series())
+            }
         }
     }
 
@@ -332,8 +496,7 @@ mod inner_mod {
             // We are still guarded by the type system.
             let out = match self.dtype() {
                 DataType::Float32 => s.f32().unwrap().pow_f32(0.5).into_series(),
-                DataType::Float64 => s.f64().unwrap().pow_f64(0.5).into_series(),
-                _ => unreachable!(),
+                _ => s.f64().unwrap().pow_f64(0.5).into_series(), //Float64 case
             };
             Ok(out)
         }
@@ -497,9 +660,21 @@ mod test {
             ],
         );
 
-        let out = ca.rolling_apply(3, &|s| s.sum_as_series()).unwrap();
+        let out = ca
+            .rolling_apply(
+                &|s| s.sum_as_series(),
+                RollingOptions {
+                    window_size: 3,
+                    min_periods: 3,
+                    ..Default::default()
+                },
+            )
+            .unwrap();
+
+        let out = out.f64().unwrap();
+
         assert_eq!(
-            Vec::from(&out),
+            Vec::from(out),
             &[
                 None,
                 None,
@@ -567,4 +742,151 @@ mod test {
             &[None, None, Some(17), Some(10), Some(20), Some(34),]
         );
     }
+
+    #[test]
+    fn test_median_quantile_types() {
+        let ca = Int32Chunked::new("foo", &[1, 2, 3, 2, 1]);
+        let rol_med = ca
+            .rolling_median(RollingOptions {
+                window_size: 2,
+                min_periods: 1,
+                ..Default::default()
+            })
+            .unwrap();
+
+        let rol_med_weighted = ca
+            .rolling_median(RollingOptions {
+                window_size: 2,
+                min_periods: 1,
+                weights: Some(vec![1.0, 2.0]),
+                ..Default::default()
+            })
+            .unwrap();
+
+        let rol_quantile = ca
+            .rolling_quantile(
+                0.3,
+                QuantileInterpolOptions::Linear,
+                RollingOptions {
+                    window_size: 2,
+                    min_periods: 1,
+                    ..Default::default()
+                },
+            )
+            .unwrap();
+
+        let rol_quantile_weighted = ca
+            .rolling_quantile(
+                0.3,
+                QuantileInterpolOptions::Linear,
+                RollingOptions {
+                    window_size: 2,
+                    min_periods: 1,
+                    weights: Some(vec![1.0, 2.0]),
+                    ..Default::default()
+                },
+            )
+            .unwrap();
+
+        assert_eq!(*rol_med.dtype(), DataType::Float64);
+        assert_eq!(*rol_med_weighted.dtype(), DataType::Float64);
+        assert_eq!(*rol_quantile.dtype(), DataType::Float64);
+        assert_eq!(*rol_quantile_weighted.dtype(), DataType::Float64);
+
+        let ca = Float32Chunked::new("foo", &[1.0, 2.0, 3.0, 2.0, 1.0]);
+        let rol_med = ca
+            .rolling_median(RollingOptions {
+                window_size: 2,
+                min_periods: 1,
+                ..Default::default()
+            })
+            .unwrap();
+
+        let rol_med_weighted = ca
+            .rolling_median(RollingOptions {
+                window_size: 2,
+                min_periods: 1,
+                weights: Some(vec![1.0, 2.0]),
+                ..Default::default()
+            })
+            .unwrap();
+
+        let rol_quantile = ca
+            .rolling_quantile(
+                0.3,
+                QuantileInterpolOptions::Linear,
+                RollingOptions {
+                    window_size: 2,
+                    min_periods: 1,
+                    ..Default::default()
+                },
+            )
+            .unwrap();
+
+        let rol_quantile_weighted = ca
+            .rolling_quantile(
+                0.3,
+                QuantileInterpolOptions::Linear,
+                RollingOptions {
+                    window_size: 2,
+                    min_periods: 1,
+                    weights: Some(vec![1.0, 2.0]),
+                    ..Default::default()
+                },
+            )
+            .unwrap();
+
+        assert_eq!(*rol_med.dtype(), DataType::Float32);
+        assert_eq!(*rol_med_weighted.dtype(), DataType::Float32);
+        assert_eq!(*rol_quantile.dtype(), DataType::Float32);
+        assert_eq!(*rol_quantile_weighted.dtype(), DataType::Float32);
+
+        let ca = Float64Chunked::new("foo", &[1.0, 2.0, 3.0, 2.0, 1.0]);
+        let rol_med = ca
+            .rolling_median(RollingOptions {
+                window_size: 2,
+                min_periods: 1,
+                ..Default::default()
+            })
+            .unwrap();
+
+        let rol_med_weighted = ca
+            .rolling_median(RollingOptions {
+                window_size: 2,
+                min_periods: 1,
+                weights: Some(vec![1.0, 2.0]),
+                ..Default::default()
+            })
+            .unwrap();
+
+        let rol_quantile = ca
+            .rolling_quantile(
+                0.3,
+                QuantileInterpolOptions::Linear,
+                RollingOptions {
+                    window_size: 2,
+                    min_periods: 1,
+                    ..Default::default()
+                },
+            )
+            .unwrap();
+
+        let rol_quantile_weighted = ca
+            .rolling_quantile(
+                0.3,
+                QuantileInterpolOptions::Linear,
+                RollingOptions {
+                    window_size: 2,
+                    min_periods: 1,
+                    weights: Some(vec![1.0, 2.0]),
+                    ..Default::default()
+                },
+            )
+            .unwrap();
+
+        assert_eq!(*rol_med.dtype(), DataType::Float64);
+        assert_eq!(*rol_med_weighted.dtype(), DataType::Float64);
+        assert_eq!(*rol_quantile.dtype(), DataType::Float64);
+        assert_eq!(*rol_quantile_weighted.dtype(), DataType::Float64);
+    }
 }
