@@ -13,11 +13,13 @@ pub use crate::chunked_array::logical::*;
 use crate::chunked_array::object::PolarsObjectSafe;
 use crate::chunked_array::ops::sort::PlIsNan;
 use crate::prelude::*;
+use crate::utils::Wrap;
 use ahash::RandomState;
 use arrow::compute::arithmetics::basic::NativeArithmetics;
 use arrow::compute::comparison::Simd8;
 use arrow::datatypes::IntegerType;
 pub use arrow::datatypes::{DataType as ArrowDataType, TimeUnit as ArrowTimeUnit};
+use arrow::error::ArrowError;
 use arrow::types::simd::Simd;
 use arrow::types::NativeType;
 use num::{Bounded, FromPrimitive, Num, NumCast, Zero};
@@ -25,6 +27,7 @@ use num::{Bounded, FromPrimitive, Num, NumCast, Zero};
 use serde::{Deserialize, Serialize};
 use std::cmp::Ordering;
 use std::fmt::{Display, Formatter};
+use std::hash::{Hash, Hasher};
 use std::ops::{Add, AddAssign, Div, Mul, Rem, Sub};
 
 pub struct Utf8Type {}
@@ -64,6 +67,7 @@ impl_polars_datatype!(Float32Type, Float32, f32);
 impl_polars_datatype!(Float64Type, Float64, f64);
 impl_polars_datatype!(DateType, Date, i32);
 impl_polars_datatype!(DatetimeType, Unknown, i64);
+impl_polars_datatype!(DurationType, Unknown, i64);
 impl_polars_datatype!(TimeType, Time, i64);
 
 impl PolarsDataType for Utf8Type {
@@ -244,6 +248,9 @@ pub enum AnyValue<'a> {
     /// in nanoseconds (64 bits).
     #[cfg(feature = "dtype-datetime")]
     Datetime(i64, TimeUnit, &'a Option<TimeZone>),
+    // A 64-bit integer representing difference between date-times in nanoseconds or milliseconds
+    #[cfg(feature = "dtype-duration")]
+    Duration(i64, TimeUnit),
     /// A 64-bit time representing the elapsed time since midnight in nanoseconds
     #[cfg(feature = "dtype-time")]
     Time(i64),
@@ -256,6 +263,29 @@ pub enum AnyValue<'a> {
     Object(&'a dyn PolarsObjectSafe),
 }
 
+impl<'a> Hash for AnyValue<'a> {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        use AnyValue::*;
+        match self {
+            Null => state.write_u64(u64::MAX / 2 + 135123),
+            Int8(v) => state.write_i8(*v),
+            Int16(v) => state.write_i16(*v),
+            Int32(v) => state.write_i32(*v),
+            Int64(v) => state.write_i64(*v),
+            UInt8(v) => state.write_u8(*v),
+            UInt16(v) => state.write_u16(*v),
+            UInt32(v) => state.write_u32(*v),
+            UInt64(v) => state.write_u64(*v),
+            Utf8(s) => state.write(s.as_bytes()),
+            Boolean(v) => state.write_u8(*v as u8),
+            List(v) => Hash::hash(&Wrap(v.clone()), state),
+            _ => unimplemented!(),
+        }
+    }
+}
+
+impl<'a> Eq for AnyValue<'a> {}
+
 impl From<f64> for AnyValue<'_> {
     fn from(a: f64) -> Self {
         AnyValue::Float64(a)
@@ -339,6 +369,15 @@ impl<'a> AnyValue<'a> {
         }
     }
 
+    pub(crate) fn into_duration(self, tu: TimeUnit) -> Self {
+        match self {
+            #[cfg(feature = "dtype-duration")]
+            AnyValue::Int64(v) => AnyValue::Duration(v, tu),
+            AnyValue::Null => AnyValue::Null,
+            dt => panic!("cannot create date from other type. dtype: {}", dt),
+        }
+    }
+
     #[cfg(feature = "dtype-time")]
     pub(crate) fn into_time(self) -> Self {
         match self {
@@ -348,6 +387,7 @@ impl<'a> AnyValue<'a> {
         }
     }
 
+    #[must_use]
     pub fn add<'b>(&self, rhs: &AnyValue<'b>) -> Self {
         use AnyValue::*;
         match (self, rhs) {
@@ -432,6 +472,7 @@ impl Display for DataType {
                 };
                 return f.write_str(&s);
             }
+            DataType::Duration(tu) => return write!(f, "duration[{}]", tu),
             DataType::Time => "time",
             DataType::List(tp) => return write!(f, "list [{}]", tp),
             #[cfg(feature = "object")]
@@ -578,6 +619,8 @@ pub enum DataType {
     /// A 64-bit date representing the elapsed time since UNIX epoch (1970-01-01)
     /// in milliseconds (64 bits).
     Datetime(TimeUnit, Option<TimeZone>),
+    // 64-bit integer representing difference between times in milliseconds or nanoseconds
+    Duration(TimeUnit),
     /// A 64-bit time representing the elapsed time since midnight in nanoseconds
     Time,
     List(Box<DataType>),
@@ -601,11 +644,13 @@ impl DataType {
     }
 
     /// Convert to the physical data type
+    #[must_use]
     pub fn to_physical(&self) -> DataType {
         use DataType::*;
         match self {
             Date => Int32,
             Datetime(_, _) => Int64,
+            Duration(_) => Int64,
             Time => Int64,
             Categorical => UInt32,
             _ => self.clone(),
@@ -630,6 +675,7 @@ impl DataType {
             Utf8 => ArrowDataType::LargeUtf8,
             Date => ArrowDataType::Date32,
             Datetime(unit, tz) => ArrowDataType::Timestamp(unit.to_arrow(), tz.clone()),
+            Duration(unit) => ArrowDataType::Duration(unit.to_arrow()),
             Time => ArrowDataType::Time64(ArrowTimeUnit::Nanosecond),
             List(dt) => ArrowDataType::LargeList(Box::new(arrow::datatypes::Field::new(
                 "",
@@ -751,6 +797,27 @@ impl Field {
     }
 }
 
+#[cfg(feature = "private")]
+pub trait IndexOfSchema {
+    fn index_of(&self, name: &str) -> arrow::error::Result<usize>;
+}
+
+impl IndexOfSchema for ArrowSchema {
+    fn index_of(&self, name: &str) -> arrow::error::Result<usize> {
+        self.fields
+            .iter()
+            .position(|f| f.name == name)
+            .ok_or_else(|| {
+                let valid_fields: Vec<String> =
+                    self.fields.iter().map(|f| f.name.clone()).collect();
+                ArrowError::InvalidArgumentError(format!(
+                    "Unable to get field named \"{}\". Valid fields: {:?}",
+                    name, valid_fields
+                ))
+            })
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Hash, Default)]
 pub struct Schema {
     fields: Vec<Field>,
@@ -812,20 +879,21 @@ impl Schema {
 
     /// Find the index of the column with the given name
     pub fn index_of(&self, name: &str) -> Result<usize> {
-        for i in 0..self.fields.len() {
-            if self.fields[i].name == name {
-                return Ok(i);
-            }
-        }
-        let valid_fields: Vec<String> = self.fields.iter().map(|f| f.name().clone()).collect();
-        Err(PolarsError::NotFound(format!(
-            "Unable to get field named \"{}\". Valid fields: {:?}",
-            name, valid_fields
-        )))
+        self.fields
+            .iter()
+            .position(|f| f.name == name)
+            .ok_or_else(|| {
+                let valid_fields: Vec<String> =
+                    self.fields.iter().map(|f| f.name.clone()).collect();
+                PolarsError::NotFound(format!(
+                    "Unable to get field named \"{}\". Valid fields: {:?}",
+                    name, valid_fields
+                ))
+            })
     }
 
     pub fn to_arrow(&self) -> ArrowSchema {
-        let fields = self
+        let fields: Vec<ArrowField> = self
             .fields
             .iter()
             .map(|f| {
@@ -854,7 +922,7 @@ impl Schema {
                 }
             })
             .collect();
-        ArrowSchema::new(fields)
+        ArrowSchema::from(fields)
     }
 
     pub fn try_merge(schemas: &[Self]) -> Result<Self> {
@@ -909,6 +977,7 @@ impl From<&ArrowDataType> for DataType {
             ArrowDataType::List(f) => DataType::List(Box::new(f.data_type().into())),
             ArrowDataType::Date32 => DataType::Date,
             ArrowDataType::Timestamp(tu, tz) => DataType::Datetime(tu.into(), tz.clone()),
+            ArrowDataType::Duration(tu) => DataType::Duration(tu.into()),
             ArrowDataType::Date64 => DataType::Datetime(TimeUnit::Milliseconds, None),
             ArrowDataType::LargeUtf8 => DataType::Utf8,
             ArrowDataType::Utf8 => DataType::Utf8,
@@ -931,14 +1000,14 @@ impl From<&ArrowDataType> for DataType {
 
 impl From<&ArrowField> for Field {
     fn from(f: &ArrowField) -> Self {
-        Field::new(f.name(), f.data_type().into())
+        Field::new(&f.name, f.data_type().into())
     }
 }
 impl From<&ArrowSchema> for Schema {
     fn from(a_schema: &ArrowSchema) -> Self {
         Schema::new(
             a_schema
-                .fields()
+                .fields
                 .iter()
                 .map(|arrow_f| arrow_f.into())
                 .collect(),
@@ -963,6 +1032,14 @@ mod test {
     #[test]
     fn test_arrow_dtypes_to_polars() {
         let dtypes = [
+            (
+                ArrowDataType::Duration(ArrowTimeUnit::Nanosecond),
+                DataType::Duration(TimeUnit::Nanoseconds),
+            ),
+            (
+                ArrowDataType::Duration(ArrowTimeUnit::Millisecond),
+                DataType::Duration(TimeUnit::Milliseconds),
+            ),
             (
                 ArrowDataType::Date64,
                 DataType::Datetime(TimeUnit::Milliseconds, None),
