@@ -1,10 +1,17 @@
+use crate::chunked_array::builder::get_list_builder;
 use crate::prelude::*;
 use crate::utils::get_supertype;
 use std::fmt::{Debug, Formatter};
 
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, Default)]
 pub struct Row<'a>(pub Vec<AnyValue<'a>>);
 
+impl<'a> Row<'a> {
+    pub fn new(values: Vec<AnyValue<'a>>) -> Self {
+        Row(values)
+    }
+}
+
 impl DataFrame {
     /// Get a row from a DataFrame. Use of this is discouraged as it will likely be slow.
     #[cfg_attr(docsrs, doc(cfg(feature = "rows")))]
@@ -50,7 +57,10 @@ impl DataFrame {
         Self::from_rows_iter_and_schema(rows.iter(), schema)
     }
 
-    fn from_rows_iter_and_schema<'a, I>(mut rows: I, schema: &Schema) -> Result<Self>
+    /// Create a new DataFrame from an iterator over rows. This should only be used when you have row wise data,
+    /// as this is a lot slower than creating the `Series` in a columnar fashion
+    #[cfg_attr(docsrs, doc(cfg(feature = "rows")))]
+    pub fn from_rows_iter_and_schema<'a, I>(mut rows: I, schema: &Schema) -> Result<Self>
     where
         I: Iterator<Item = &'a Row<'a>>,
     {
@@ -60,14 +70,14 @@ impl DataFrame {
             .fields()
             .iter()
             .map(|fld| {
-                let buf: Buffer = (fld.data_type(), capacity).into();
+                let buf: AnyValueBuffer = (fld.data_type(), capacity).into();
                 buf
             })
             .collect();
 
         rows.try_for_each::<_, Result<()>>(|row| {
             for (value, buf) in row.0.iter().zip(&mut buffers) {
-                buf.add(value.clone())?
+                buf.add_falible(value.clone())?
             }
             Ok(())
         })?;
@@ -204,6 +214,7 @@ impl<'a> From<&AnyValue<'a>> for Field {
             Datetime(_, tu, tz) => Field::new("", DataType::Datetime(*tu, (*tz).clone())),
             #[cfg(feature = "dtype-time")]
             Time(_) => Field::new("", DataType::Time),
+            List(s) => Field::new("", DataType::List(Box::new(s.dtype().clone()))),
             _ => unimplemented!(),
         }
     }
@@ -225,7 +236,7 @@ impl From<&Row<'_>> for Schema {
     }
 }
 
-pub(crate) enum Buffer {
+pub(crate) enum AnyValueBuffer {
     Boolean(BooleanChunkedBuilder),
     Int32(PrimitiveChunkedBuilder<Int32Type>),
     Int64(PrimitiveChunkedBuilder<Int64Type>),
@@ -244,11 +255,12 @@ pub(crate) enum Buffer {
     Float32(PrimitiveChunkedBuilder<Float32Type>),
     Float64(PrimitiveChunkedBuilder<Float64Type>),
     Utf8(Utf8ChunkedBuilder),
+    List(Box<dyn ListBuilderTrait>),
 }
 
-impl Debug for Buffer {
+impl Debug for AnyValueBuffer {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        use Buffer::*;
+        use AnyValueBuffer::*;
         match self {
             Boolean(_) => f.write_str("boolean"),
             Int32(_) => f.write_str("i32"),
@@ -264,13 +276,14 @@ impl Debug for Buffer {
             Float32(_) => f.write_str("f32"),
             Float64(_) => f.write_str("f64"),
             Utf8(_) => f.write_str("utf8"),
+            List(_) => f.write_str("list"),
         }
     }
 }
 
-impl Buffer {
-    fn add(&mut self, val: AnyValue) -> Result<()> {
-        use Buffer::*;
+impl AnyValueBuffer {
+    pub(crate) fn add(&mut self, val: AnyValue) -> Option<()> {
+        use AnyValueBuffer::*;
         match (self, val) {
             (Boolean(builder), AnyValue::Boolean(v)) => builder.append_value(v),
             (Boolean(builder), AnyValue::Null) => builder.append_null(),
@@ -284,22 +297,37 @@ impl Buffer {
             (UInt64(builder), AnyValue::Null) => builder.append_null(),
             #[cfg(feature = "dtype-date")]
             (Date(builder), AnyValue::Null) => builder.append_null(),
+            #[cfg(feature = "dtype-date")]
+            (Date(builder), AnyValue::Date(v)) => builder.append_value(v),
+            #[cfg(feature = "dtype-datetime")]
+            (Datetime(builder, _, _), AnyValue::Null) => builder.append_null(),
             #[cfg(feature = "dtype-datetime")]
             (Datetime(builder, _, _), AnyValue::Datetime(v, _, _)) => builder.append_value(v),
             #[cfg(feature = "dtype-time")]
             (Time(builder), AnyValue::Time(v)) => builder.append_value(v),
+            #[cfg(feature = "dtype-time")]
+            (Time(builder), AnyValue::Null) => builder.append_null(),
+            (Float32(builder), AnyValue::Float32(v)) => builder.append_value(v),
             (Float32(builder), AnyValue::Null) => builder.append_null(),
             (Float64(builder), AnyValue::Float64(v)) => builder.append_value(v),
+            (Float64(builder), AnyValue::Null) => builder.append_null(),
             (Utf8(builder), AnyValue::Utf8(v)) => builder.append_value(v),
             (Utf8(builder), AnyValue::Null) => builder.append_null(),
-            (buf, val) => return Err(PolarsError::ValueError(format!("Could not append {:?} to builder {:?}; make sure that all rows have the same schema.", val, std::mem::discriminant(buf)).into()))
+            (List(builder), AnyValue::List(v)) => builder.append_series(&v),
+            (List(builder), AnyValue::Null) => builder.append_null(),
+            _ => return None,
         };
-
-        Ok(())
+        Some(())
     }
 
-    fn into_series(self) -> Series {
-        use Buffer::*;
+    pub(crate) fn add_falible(&mut self, val: AnyValue) -> Result<()> {
+        self.add(val.clone()).ok_or_else(|| {
+            PolarsError::ValueError(format!("Could not append {:?} to builder; make sure that all rows have the same schema.", val).into())
+        })
+    }
+
+    pub(crate) fn into_series(self) -> Series {
+        use AnyValueBuffer::*;
         match self {
             Boolean(b) => b.finish().into_series(),
             Int32(b) => b.finish().into_series(),
@@ -315,32 +343,34 @@ impl Buffer {
             Float32(b) => b.finish().into_series(),
             Float64(b) => b.finish().into_series(),
             Utf8(b) => b.finish().into_series(),
+            List(mut b) => b.finish().into_series(),
         }
     }
 }
 
 // datatype and length
-impl From<(&DataType, usize)> for Buffer {
+impl From<(&DataType, usize)> for AnyValueBuffer {
     fn from(a: (&DataType, usize)) -> Self {
         let (dt, len) = a;
         use DataType::*;
         match dt {
-            Boolean => Buffer::Boolean(BooleanChunkedBuilder::new("", len)),
-            Int32 => Buffer::Int32(PrimitiveChunkedBuilder::new("", len)),
-            Int64 => Buffer::Int64(PrimitiveChunkedBuilder::new("", len)),
-            UInt32 => Buffer::UInt32(PrimitiveChunkedBuilder::new("", len)),
-            UInt64 => Buffer::UInt64(PrimitiveChunkedBuilder::new("", len)),
+            Boolean => AnyValueBuffer::Boolean(BooleanChunkedBuilder::new("", len)),
+            Int32 => AnyValueBuffer::Int32(PrimitiveChunkedBuilder::new("", len)),
+            Int64 => AnyValueBuffer::Int64(PrimitiveChunkedBuilder::new("", len)),
+            UInt32 => AnyValueBuffer::UInt32(PrimitiveChunkedBuilder::new("", len)),
+            UInt64 => AnyValueBuffer::UInt64(PrimitiveChunkedBuilder::new("", len)),
             #[cfg(feature = "dtype-date")]
-            Date => Buffer::Date(PrimitiveChunkedBuilder::new("", len)),
+            Date => AnyValueBuffer::Date(PrimitiveChunkedBuilder::new("", len)),
             #[cfg(feature = "dtype-datetime")]
             Datetime(tu, tz) => {
-                Buffer::Datetime(PrimitiveChunkedBuilder::new("", len), *tu, tz.clone())
+                AnyValueBuffer::Datetime(PrimitiveChunkedBuilder::new("", len), *tu, tz.clone())
             }
             #[cfg(feature = "dtype-time")]
-            Time => Buffer::Time(PrimitiveChunkedBuilder::new("", len)),
-            Float32 => Buffer::Float32(PrimitiveChunkedBuilder::new("", len)),
-            Float64 => Buffer::Float64(PrimitiveChunkedBuilder::new("", len)),
-            Utf8 => Buffer::Utf8(Utf8ChunkedBuilder::new("", len, len * 5)),
+            Time => AnyValueBuffer::Time(PrimitiveChunkedBuilder::new("", len)),
+            Float32 => AnyValueBuffer::Float32(PrimitiveChunkedBuilder::new("", len)),
+            Float64 => AnyValueBuffer::Float64(PrimitiveChunkedBuilder::new("", len)),
+            Utf8 => AnyValueBuffer::Utf8(Utf8ChunkedBuilder::new("", len, len * 5)),
+            List(inner) => AnyValueBuffer::List(get_list_builder(inner, len * 10, len, "")),
             _ => unimplemented!(),
         }
     }
