@@ -11,8 +11,6 @@ use crate::chunked_array::{
     AsSinglePtr, ChunkIdIter,
 };
 use crate::fmt::FmtList;
-#[cfg(feature = "pivot")]
-use crate::frame::groupby::pivot::*;
 use crate::frame::groupby::*;
 use crate::frame::hash_join::{HashJoin, ZipOuterJoinColumn};
 use crate::prelude::*;
@@ -21,8 +19,7 @@ use crate::series::arithmetic::checked::NumOpsDispatchChecked;
 use crate::series::implementations::SeriesWrap;
 use ahash::RandomState;
 use arrow::array::ArrayRef;
-#[cfg(feature = "object")]
-use std::any::Any;
+use polars_arrow::prelude::QuantileInterpolOptions;
 use std::borrow::Cow;
 use std::ops::Deref;
 
@@ -71,47 +68,10 @@ impl private::PrivateSeries for SeriesWrap<CategoricalChunked> {
         self.0.vec_hash_combine(build_hasher, hashes)
     }
 
-    fn agg_first(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-        self.0.agg_first(groups)
-    }
-
-    fn agg_last(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-        self.0.agg_last(groups)
-    }
-
-    fn agg_n_unique(&self, groups: &[(u32, Vec<u32>)]) -> Option<UInt32Chunked> {
-        self.0.agg_n_unique(groups)
-    }
-
-    fn agg_list(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+    fn agg_list(&self, groups: &GroupsProxy) -> Option<Series> {
         self.0.agg_list(groups)
     }
 
-    #[cfg(feature = "lazy")]
-    fn agg_valid_count(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
-        self.0.agg_valid_count(groups)
-    }
-
-    #[cfg(feature = "pivot")]
-    fn pivot<'a>(
-        &self,
-        pivot_series: &'a Series,
-        keys: Vec<Series>,
-        groups: &[(u32, Vec<u32>)],
-        agg_type: PivotAgg,
-    ) -> Result<DataFrame> {
-        self.0.pivot(pivot_series, keys, groups, agg_type)
-    }
-
-    #[cfg(feature = "pivot")]
-    fn pivot_count<'a>(
-        &self,
-        pivot_series: &'a Series,
-        keys: Vec<Series>,
-        groups: &[(u32, Vec<u32>)],
-    ) -> Result<DataFrame> {
-        self.0.pivot_count(pivot_series, keys, groups)
-    }
     fn hash_join_inner(&self, other: &Series) -> Vec<(u32, u32)> {
         HashJoin::hash_join_inner(&self.0, other.as_ref().as_ref())
     }
@@ -132,19 +92,14 @@ impl private::PrivateSeries for SeriesWrap<CategoricalChunked> {
             .cast(&DataType::Categorical)
             .unwrap()
     }
-    fn group_tuples(&self, multithreaded: bool) -> GroupTuples {
-        IntoGroupTuples::group_tuples(&self.0, multithreaded)
+    fn group_tuples(&self, multithreaded: bool) -> GroupsProxy {
+        IntoGroupsProxy::group_tuples(&self.0, multithreaded)
     }
 
     #[cfg(feature = "sort_multiple")]
     fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<UInt32Chunked> {
         self.0.argsort_multiple(by, reverse)
     }
-
-    fn str_value(&self, index: usize) -> Cow<str> {
-        // get AnyValue
-        Cow::Owned(format!("{}", self.get(index)))
-    }
 }
 
 impl SeriesTrait for SeriesWrap<CategoricalChunked> {
@@ -257,14 +212,6 @@ impl SeriesTrait for SeriesWrap<CategoricalChunked> {
         ChunkOps::rechunk(&self.0).into_series()
     }
 
-    fn head(&self, length: Option<usize>) -> Series {
-        self.0.head(length).into_series()
-    }
-
-    fn tail(&self, length: Option<usize>) -> Series {
-        self.0.tail(length).into_series()
-    }
-
     fn expand_at_index(&self, index: usize, length: usize) -> Series {
         ChunkExpandAtIndex::expand_at_index(&self.0, index, length).into_series()
     }
@@ -401,10 +348,6 @@ impl SeriesTrait for SeriesWrap<CategoricalChunked> {
         self.0.is_first()
     }
 
-    #[cfg(feature = "object")]
-    fn as_any(&self) -> &dyn Any {
-        &self.0
-    }
     #[cfg(feature = "mode")]
     fn mode(&self) -> Result<Series> {
         Ok(self.0.mode()?.into_series())
