@@ -16,13 +16,10 @@ use crate::chunked_array::{
     AsSinglePtr, ChunkIdIter,
 };
 use crate::fmt::FmtList;
-#[cfg(feature = "pivot")]
-use crate::frame::groupby::pivot::*;
 use crate::frame::{groupby::*, hash_join::*};
 use crate::prelude::*;
 use ahash::RandomState;
-#[cfg(feature = "object")]
-use std::any::Any;
+use polars_arrow::prelude::QuantileInterpolOptions;
 use std::borrow::Cow;
 use std::ops::{Deref, DerefMut};
 
@@ -59,11 +56,6 @@ macro_rules! impl_dyn_series {
                 self.0.cummin(reverse).$into_logical().into_series()
             }
 
-            #[cfg(feature = "cum_agg")]
-            fn _cumsum(&self, _reverse: bool) -> Series {
-                panic!("cannot sum logical")
-            }
-
             #[cfg(feature = "asof_join")]
             fn join_asof(&self, other: &Series) -> Result<Vec<Option<u32>>> {
                 let other = other.to_physical_repr();
@@ -99,51 +91,39 @@ macro_rules! impl_dyn_series {
                 self.0.vec_hash_combine(build_hasher, hashes)
             }
 
-            fn agg_mean(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_mean(&self, _groups: &GroupsProxy) -> Option<Series> {
                 // does not make sense on logical
                 None
             }
 
-            fn agg_min(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_min(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0
                     .agg_min(groups)
                     .map(|ca| ca.$into_logical().into_series())
             }
 
-            fn agg_max(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_max(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0
                     .agg_max(groups)
                     .map(|ca| ca.$into_logical().into_series())
             }
 
-            fn agg_sum(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_sum(&self, _groups: &GroupsProxy) -> Option<Series> {
                 // does not make sense on logical
                 None
             }
 
-            fn agg_first(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-                self.0.agg_first(groups).$into_logical().into_series()
-            }
-
-            fn agg_last(&self, groups: &[(u32, Vec<u32>)]) -> Series {
-                self.0.agg_last(groups).$into_logical().into_series()
-            }
-
-            fn agg_std(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_std(&self, _groups: &GroupsProxy) -> Option<Series> {
                 // does not make sense on logical
                 None
             }
 
-            fn agg_var(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_var(&self, _groups: &GroupsProxy) -> Option<Series> {
                 // does not make sense on logical
                 None
             }
 
-            fn agg_n_unique(&self, groups: &[(u32, Vec<u32>)]) -> Option<UInt32Chunked> {
-                self.0.agg_n_unique(groups)
-            }
-
-            fn agg_list(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_list(&self, groups: &GroupsProxy) -> Option<Series> {
                 // we cannot cast and dispatch as the inner type of the list would be incorrect
                 self.0.agg_list(groups).map(|s| {
                     s.cast(&DataType::List(Box::new(self.dtype().clone())))
@@ -153,7 +133,7 @@ macro_rules! impl_dyn_series {
 
             fn agg_quantile(
                 &self,
-                groups: &[(u32, Vec<u32>)],
+                groups: &GroupsProxy,
                 quantile: f64,
                 interpol: QuantileInterpolOptions,
             ) -> Option<Series> {
@@ -162,36 +142,12 @@ macro_rules! impl_dyn_series {
                     .map(|s| s.$into_logical().into_series())
             }
 
-            fn agg_median(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
+            fn agg_median(&self, groups: &GroupsProxy) -> Option<Series> {
                 self.0
                     .agg_median(groups)
                     .map(|s| s.$into_logical().into_series())
             }
-            #[cfg(feature = "lazy")]
-            fn agg_valid_count(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
-                self.0.agg_valid_count(groups)
-            }
 
-            #[cfg(feature = "pivot")]
-            fn pivot<'a>(
-                &self,
-                pivot_series: &'a Series,
-                keys: Vec<Series>,
-                groups: &[(u32, Vec<u32>)],
-                agg_type: PivotAgg,
-            ) -> Result<DataFrame> {
-                self.0.pivot(pivot_series, keys, groups, agg_type)
-            }
-
-            #[cfg(feature = "pivot")]
-            fn pivot_count<'a>(
-                &self,
-                pivot_series: &'a Series,
-                keys: Vec<Series>,
-                groups: &[(u32, Vec<u32>)],
-            ) -> Result<DataFrame> {
-                self.0.pivot_count(pivot_series, keys, groups)
-            }
             fn hash_join_inner(&self, other: &Series) -> Vec<(u32, u32)> {
                 let other = other.to_physical_repr().into_owned();
                 self.0.hash_join_inner(&other.as_ref().as_ref())
@@ -218,9 +174,10 @@ macro_rules! impl_dyn_series {
             fn subtract(&self, rhs: &Series) -> Result<Series> {
                 match (self.dtype(), rhs.dtype()) {
                     (DataType::Date, DataType::Date) => {
-                        let lhs = self.cast(&DataType::Int32).unwrap();
-                        let rhs = rhs.cast(&DataType::Int32).unwrap();
-                        Ok(lhs.subtract(&rhs)?.$into_logical().into_series())
+                        let dt = DataType::Datetime(TimeUnit::Milliseconds, None);
+                        let lhs = self.cast(&dt)?;
+                        let rhs = rhs.cast(&dt)?;
+                        lhs.subtract(&rhs)
                     }
                     (dtl, dtr) => Err(PolarsError::ComputeError(
                         format!(
@@ -231,10 +188,16 @@ macro_rules! impl_dyn_series {
                     )),
                 }
             }
-            fn add_to(&self, _rhs: &Series) -> Result<Series> {
-                Err(PolarsError::ComputeError(
-                    "cannot do addition on logical".into(),
-                ))
+            fn add_to(&self, rhs: &Series) -> Result<Series> {
+                match (self.dtype(), rhs.dtype()) {
+                    (dtl, dtr) => Err(PolarsError::ComputeError(
+                        format!(
+                            "cannot do addition on these date types: {:?}, {:?}",
+                            dtl, dtr
+                        )
+                        .into(),
+                    )),
+                }
             }
             fn multiply(&self, _rhs: &Series) -> Result<Series> {
                 Err(PolarsError::ComputeError(
@@ -251,18 +214,13 @@ macro_rules! impl_dyn_series {
                     "cannot do remainder operation on logical".into(),
                 ))
             }
-            fn group_tuples(&self, multithreaded: bool) -> GroupTuples {
+            fn group_tuples(&self, multithreaded: bool) -> GroupsProxy {
                 self.0.group_tuples(multithreaded)
             }
             #[cfg(feature = "sort_multiple")]
             fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<UInt32Chunked> {
                 self.0.deref().argsort_multiple(by, reverse)
             }
-
-            fn str_value(&self, index: usize) -> Cow<str> {
-                // get AnyValue
-                Cow::Owned(format!("{}", self.get(index)))
-            }
         }
 
         impl SeriesTrait for SeriesWrap<$ca> {
@@ -400,14 +358,6 @@ macro_rules! impl_dyn_series {
                 self.0.rechunk().$into_logical().into_series()
             }
 
-            fn head(&self, length: Option<usize>) -> Series {
-                self.0.head(length).$into_logical().into_series()
-            }
-
-            fn tail(&self, length: Option<usize>) -> Series {
-                self.0.tail(length).$into_logical().into_series()
-            }
-
             fn expand_at_index(&self, index: usize, length: usize) -> Series {
                 self.0
                     .expand_at_index(index, length)
@@ -628,11 +578,6 @@ macro_rules! impl_dyn_series {
                 self.0.is_first()
             }
 
-            #[cfg(feature = "object")]
-            fn as_any(&self) -> &dyn Any {
-                &self.0
-            }
-
             #[cfg(feature = "mode")]
             fn mode(&self) -> Result<Series> {
                 self.0.mode().map(|ca| ca.$into_logical().into_series())
@@ -681,7 +626,9 @@ mod test {
         let s = Series::new("foo", &[1, 2, 3]);
         let s = s.cast(&DataType::Datetime(TimeUnit::Nanoseconds, None))?;
 
-        let l = s.agg_list(&[(0, vec![0, 1, 2])]).unwrap();
+        let l = s
+            .agg_list(&GroupsProxy::Idx(vec![(0, vec![0, 1, 2])]))
+            .unwrap();
 
         match l.dtype() {
             DataType::List(inner) => {
@@ -706,19 +653,19 @@ mod test {
 
         let df = DataFrame::new(vec![s, s1])?;
 
-        let out = df.left_join(&df.clone(), "bar", "bar")?;
+        let out = df.left_join(&df.clone(), ["bar"], ["bar"])?;
         assert!(matches!(
             out.column("bar")?.dtype(),
             DataType::Datetime(TimeUnit::Nanoseconds, None)
         ));
 
-        let out = df.inner_join(&df.clone(), "bar", "bar")?;
+        let out = df.inner_join(&df.clone(), ["bar"], ["bar"])?;
         assert!(matches!(
             out.column("bar")?.dtype(),
             DataType::Datetime(TimeUnit::Nanoseconds, None)
         ));
 
-        let out = df.outer_join(&df.clone(), "bar", "bar")?;
+        let out = df.outer_join(&df.clone(), ["bar"], ["bar"])?;
         assert!(matches!(
             out.column("bar")?.dtype(),
             DataType::Datetime(TimeUnit::Nanoseconds, None)
@@ -735,7 +682,7 @@ mod test {
         let out = s.subtract(&s)?;
         assert!(matches!(
             out.dtype(),
-            DataType::Datetime(TimeUnit::Nanoseconds, None)
+            DataType::Duration(TimeUnit::Nanoseconds)
         ));
 
         let mut a = s.clone();
@@ -746,6 +693,7 @@ mod test {
     }
 
     #[test]
+    #[cfg(feature = "dtype-datetime")]
     fn test_arithmetic_dispatch() {
         let s = Int64Chunked::new("", &[1, 2, 3])
             .into_datetime(TimeUnit::Nanoseconds, None)
@@ -804,4 +752,33 @@ mod test {
             &DataType::Datetime(TimeUnit::Nanoseconds, None)
         );
     }
+
+    #[test]
+    #[cfg(feature = "dtype-duration")]
+    fn test_duration() -> Result<()> {
+        let a = Int64Chunked::new("", &[1, 2, 3])
+            .into_datetime(TimeUnit::Nanoseconds, None)
+            .into_series();
+        let b = Int64Chunked::new("", &[2, 3, 4])
+            .into_datetime(TimeUnit::Nanoseconds, None)
+            .into_series();
+        let c = Int64Chunked::new("", &[1, 1, 1])
+            .into_duration(TimeUnit::Nanoseconds)
+            .into_series();
+        assert_eq!(
+            *b.subtract(&a)?.dtype(),
+            DataType::Duration(TimeUnit::Nanoseconds)
+        );
+        assert_eq!(
+            *a.add_to(&c)?.dtype(),
+            DataType::Datetime(TimeUnit::Nanoseconds, None)
+        );
+        assert_eq!(
+            b.subtract(&a)?,
+            Int64Chunked::full("", 1, a.len())
+                .into_duration(TimeUnit::Nanoseconds)
+                .into_series()
+        );
+        Ok(())
+    }
 }
