@@ -1,12 +1,11 @@
+#[cfg(feature = "ipc")]
+use crate::logical_plan::LpScanOptions;
 #[cfg(feature = "parquet")]
 use crate::logical_plan::ParquetOptions;
-#[cfg(feature = "ipc")]
-use crate::logical_plan::ScanOptions;
 use crate::logical_plan::{det_melt_schema, Context, CsvParserOptions};
 use crate::prelude::*;
 use crate::utils::{aexprs_to_schema, PushNode};
 use ahash::RandomState;
-use polars_core::frame::groupby::DynamicGroupOptions;
 use polars_core::prelude::*;
 use polars_utils::arena::{Arena, Node};
 use std::collections::HashSet;
@@ -26,7 +25,7 @@ pub enum ALogicalPlan {
     Slice {
         input: Node,
         offset: i64,
-        len: usize,
+        len: u32,
     },
     Selection {
         input: Node,
@@ -49,7 +48,7 @@ pub enum ALogicalPlan {
         schema: SchemaRef,
         // schema of the projected file
         output_schema: Option<SchemaRef>,
-        options: ScanOptions,
+        options: LpScanOptions,
         predicate: Option<Node>,
         aggregate: Vec<Node>,
     },
@@ -99,7 +98,7 @@ pub enum ALogicalPlan {
         schema: SchemaRef,
         apply: Option<Arc<dyn DataFrameUdf>>,
         maintain_order: bool,
-        dynamic_options: Option<DynamicGroupOptions>,
+        options: GroupbyOptions,
     },
     Join {
         input_left: Node,
@@ -130,6 +129,7 @@ pub enum ALogicalPlan {
     },
     Union {
         inputs: Vec<Node>,
+        options: UnionOptions,
     },
 }
 
@@ -186,90 +186,18 @@ impl ALogicalPlan {
             },
         }
     }
-
-    /// Check ALogicalPlan equality. The nodes may differ.
-    ///
-    /// For instance: there can be two columns "foo" in the memory arena. These are equal,
-    /// but would have different node values.
-    #[cfg(feature = "private")]
-    pub(crate) fn eq(
-        node_left: Node,
-        node_right: Node,
-        lp_arena: &Arena<ALogicalPlan>,
-        expr_arena: &Arena<AExpr>,
-    ) -> bool {
-        use crate::logical_plan::iterator::ArenaLpIter;
-        use std::fs::canonicalize;
-
-        let cmp = |(node_left, node_right)| {
-            use ALogicalPlan::*;
-            match (lp_arena.get(node_left), lp_arena.get(node_right)) {
-                #[cfg(feature = "csv-file")]
-                (CsvScan { path: path_a, .. }, CsvScan { path: path_b, .. }) => {
-                    canonicalize(path_a).unwrap() == canonicalize(path_b).unwrap()
-                }
-                #[cfg(feature = "parquet")]
-                (ParquetScan { path: path_a, .. }, ParquetScan { path: path_b, .. }) => {
-                    canonicalize(path_a).unwrap() == canonicalize(path_b).unwrap()
-                }
-                #[cfg(feature = "ipc")]
-                (IpcScan { path: path_a, .. }, IpcScan { path: path_b, .. }) => {
-                    canonicalize(path_a).unwrap() == canonicalize(path_b).unwrap()
-                }
-                (DataFrameScan { df: df_a, .. }, DataFrameScan { df: df_b, .. }) => {
-                    df_a.ptr_equal(df_b)
-                }
-                // the following don't affect the schema, but do affect the # of rows or the row order.
-                (Selection { predicate: l, .. }, Selection { predicate: r, .. }) => {
-                    AExpr::eq(*l, *r, expr_arena)
-                }
-                (
-                    Sort {
-                        by_column: l,
-                        reverse: r_l,
-                        ..
-                    },
-                    Sort {
-                        by_column: r,
-                        reverse: r_r,
-                        ..
-                    },
-                ) => l == r && r_l == r_r,
-                (Explode { columns: l, .. }, Explode { columns: r, .. }) => l == r,
-                (
-                    Distinct {
-                        maintain_order: l1,
-                        subset: l2,
-                        ..
-                    },
-                    Distinct {
-                        maintain_order: r1,
-                        subset: r2,
-                        ..
-                    },
-                ) => l1 == r1 && l2 == r2,
-                (a, b) => {
-                    std::mem::discriminant(a) == std::mem::discriminant(b)
-                        && a.schema(lp_arena) == b.schema(lp_arena)
-                }
-            }
-        };
-
-        lp_arena
-            .iter(node_left)
-            .zip(lp_arena.iter(node_right))
-            .map(|(tpll, tplr)| (tpll.0, tplr.0))
-            .all(cmp)
-    }
 }
 
 impl ALogicalPlan {
     /// Takes the expressions of an LP node and the inputs of that node and reconstruct
-    pub fn from_exprs_and_input(&self, mut exprs: Vec<Node>, inputs: Vec<Node>) -> ALogicalPlan {
+    pub fn with_exprs_and_input(&self, mut exprs: Vec<Node>, inputs: Vec<Node>) -> ALogicalPlan {
         use ALogicalPlan::*;
 
         match self {
-            Union { .. } => Union { inputs },
+            Union { options, .. } => Union {
+                inputs,
+                options: *options,
+            },
             Melt {
                 id_vars,
                 value_vars,
@@ -305,7 +233,7 @@ impl ALogicalPlan {
                 schema,
                 apply,
                 maintain_order,
-                dynamic_options,
+                options: dynamic_options,
                 ..
             } => Aggregate {
                 input: inputs[0],
@@ -314,7 +242,7 @@ impl ALogicalPlan {
                 schema: schema.clone(),
                 apply: apply.clone(),
                 maintain_order: *maintain_order,
-                dynamic_options: dynamic_options.clone(),
+                options: dynamic_options.clone(),
             },
             Join {
                 schema,
@@ -712,9 +640,9 @@ impl<'a> ALogicalPlanBuilder<'a> {
         aggs: Vec<Node>,
         apply: Option<Arc<dyn DataFrameUdf>>,
         maintain_order: bool,
-        dynamic_options: Option<DynamicGroupOptions>,
+        options: GroupbyOptions,
     ) -> Self {
-        debug_assert!(!(keys.is_empty() && dynamic_options.is_none()));
+        debug_assert!(!(keys.is_empty() && options.dynamic.is_none()));
         let current_schema = self.schema();
         // TODO! add this line if LogicalPlan is dropped in favor of ALogicalPlan
         // let aggs = rewrite_projections(aggs, current_schema);
@@ -732,7 +660,7 @@ impl<'a> ALogicalPlanBuilder<'a> {
             schema: Arc::new(schema),
             apply,
             maintain_order,
-            dynamic_options,
+            options,
         };
         let root = self.lp_arena.add(lp);
         Self::new(root, self.expr_arena, self.lp_arena)
@@ -774,7 +702,7 @@ impl<'a> ALogicalPlanBuilder<'a> {
             let name = f.name();
             if !right_names.contains(name.as_str()) {
                 if names.contains(name.as_str()) {
-                    let new_name = format!("{}_right", name);
+                    let new_name = format!("{}{}", name, options.suffix.as_ref());
                     let field = Field::new(&new_name, f.data_type().clone());
                     fields.push(field)
                 } else {
