@@ -106,17 +106,6 @@ pub(crate) fn to_aexpr(expr: Expr, arena: &mut Arena<AExpr>) -> Node {
             output_type,
             options,
         },
-        Expr::BinaryFunction {
-            input_a,
-            input_b,
-            function,
-            output_field,
-        } => AExpr::BinaryFunction {
-            input_a: to_aexpr(*input_a, arena),
-            input_b: to_aexpr(*input_b, arena),
-            function,
-            output_field,
-        },
         Expr::Shift { input, periods } => AExpr::Shift {
             input: to_aexpr(*input, arena),
             periods,
@@ -157,12 +146,12 @@ pub(crate) fn to_alp(
     lp_arena: &mut Arena<ALogicalPlan>,
 ) -> Node {
     let v = match lp {
-        LogicalPlan::Union { inputs } => {
+        LogicalPlan::Union { inputs, options } => {
             let inputs = inputs
                 .into_iter()
                 .map(|lp| to_alp(lp, expr_arena, lp_arena))
                 .collect();
-            ALogicalPlan::Union { inputs }
+            ALogicalPlan::Union { inputs, options }
         }
         LogicalPlan::Selection { input, predicate } => {
             let i = to_alp(*input, expr_arena, lp_arena);
@@ -313,7 +302,7 @@ pub(crate) fn to_alp(
             schema,
             apply,
             maintain_order,
-            dynamic_options,
+            options,
         } => {
             let i = to_alp(*input, expr_arena, lp_arena);
             let aggs_new = aggs.into_iter().map(|x| to_aexpr(x, expr_arena)).collect();
@@ -329,7 +318,7 @@ pub(crate) fn to_alp(
                 schema,
                 apply,
                 maintain_order,
-                dynamic_options,
+                options,
             }
         }
         LogicalPlan::Join {
@@ -589,17 +578,6 @@ pub(crate) fn node_to_exp(node: Node, expr_arena: &Arena<AExpr>) -> Expr {
             output_type,
             options,
         },
-        AExpr::BinaryFunction {
-            input_a,
-            input_b,
-            function,
-            output_field,
-        } => Expr::BinaryFunction {
-            input_a: Box::new(node_to_exp(input_a, expr_arena)),
-            input_b: Box::new(node_to_exp(input_b, expr_arena)),
-            function,
-            output_field,
-        },
         AExpr::Window {
             function,
             partition_by,
@@ -642,12 +620,12 @@ pub(crate) fn node_to_lp(
     let lp = std::mem::take(lp);
 
     match lp {
-        ALogicalPlan::Union { inputs } => {
+        ALogicalPlan::Union { inputs, options } => {
             let inputs = inputs
                 .into_iter()
                 .map(|node| node_to_lp(node, expr_arena, lp_arena))
                 .collect();
-            LogicalPlan::Union { inputs }
+            LogicalPlan::Union { inputs, options }
         }
         ALogicalPlan::Slice { input, offset, len } => {
             let lp = node_to_lp(input, expr_arena, lp_arena);
@@ -776,7 +754,7 @@ pub(crate) fn node_to_lp(
             schema,
             apply,
             maintain_order,
-            dynamic_options,
+            options: dynamic_options,
         } => {
             let i = node_to_lp(input, expr_arena, lp_arena);
 
@@ -787,7 +765,7 @@ pub(crate) fn node_to_lp(
                 schema,
                 apply,
                 maintain_order,
-                dynamic_options,
+                options: dynamic_options,
             }
         }
         ALogicalPlan::Join {
