@@ -1,12 +1,13 @@
 use super::expressions as phys_expr;
 use crate::logical_plan::Context;
 use crate::physical_plan::executors::groupby_dynamic::GroupByDynamicExec;
+use crate::physical_plan::executors::groupby_rolling::GroupByRollingExec;
 #[cfg(feature = "ipc")]
 use crate::physical_plan::executors::scan::IpcExec;
 use crate::physical_plan::executors::union::UnionExec;
 use crate::prelude::shift::ShiftExpr;
 use crate::prelude::*;
-use crate::utils::expr_to_root_column_name;
+use crate::utils::{expr_to_root_column_name, has_window_aexpr};
 use crate::{
     logical_plan::iterator::ArenaExprIter,
     utils::{aexpr_to_root_names, aexpr_to_root_nodes, agg_source_paths, has_aexpr},
@@ -15,7 +16,7 @@ use ahash::RandomState;
 use polars_core::prelude::*;
 use polars_core::{frame::groupby::GroupByMethod, utils::parallel_op_series};
 #[cfg(any(feature = "parquet", feature = "csv-file", feature = "ipc"))]
-use polars_io::ScanAggregation;
+use polars_io::aggregations::ScanAggregation;
 use std::collections::HashSet;
 use std::sync::Arc;
 
@@ -98,12 +99,12 @@ impl DefaultPlanner {
         use ALogicalPlan::*;
         let logical_plan = lp_arena.take(root);
         match logical_plan {
-            Union { inputs } => {
+            Union { inputs, options } => {
                 let inputs = inputs
                     .into_iter()
                     .map(|node| self.create_physical_plan(node, lp_arena, expr_arena))
                     .collect::<Result<Vec<_>>>()?;
-                Ok(Box::new(UnionExec { inputs }))
+                Ok(Box::new(UnionExec { inputs, options }))
             }
             Melt {
                 input,
@@ -195,12 +196,14 @@ impl DefaultPlanner {
                 schema: _schema,
                 ..
             } => {
+                let has_windows = expr.iter().any(|node| has_window_aexpr(*node, expr_arena));
                 let input = self.create_physical_plan(input, lp_arena, expr_arena)?;
                 let phys_expr =
                     self.create_physical_expressions(&expr, Context::Default, expr_arena)?;
                 Ok(Box::new(ProjectionExec {
                     input,
                     expr: phys_expr,
+                    has_windows,
                     #[cfg(test)]
                     schema: _schema,
                 }))
@@ -211,12 +214,14 @@ impl DefaultPlanner {
                 schema: _schema,
                 ..
             } => {
+                let has_windows = expr.iter().any(|node| has_window_aexpr(*node, expr_arena));
                 let input = self.create_physical_plan(input, lp_arena, expr_arena)?;
                 let phys_expr =
                     self.create_physical_expressions(&expr, Context::Default, expr_arena)?;
                 Ok(Box::new(ProjectionExec {
                     input,
                     expr: phys_expr,
+                    has_windows,
                     #[cfg(test)]
                     schema: _schema,
                 }))
@@ -227,6 +232,14 @@ impl DefaultPlanner {
                 selection,
                 ..
             } => {
+                let has_windows = if let Some(projection) = &projection {
+                    projection
+                        .iter()
+                        .any(|node| has_window_aexpr(*node, expr_arena))
+                } else {
+                    false
+                };
+
                 let selection = selection
                     .map(|pred| self.create_physical_expr(pred, Context::Default, expr_arena))
                     .map_or(Ok(None), |v| v.map(Some))?;
@@ -235,7 +248,12 @@ impl DefaultPlanner {
                         self.create_physical_expressions(&proj, Context::Default, expr_arena)
                     })
                     .map_or(Ok(None), |v| v.map(Some))?;
-                Ok(Box::new(DataFrameExec::new(df, projection, selection)))
+                Ok(Box::new(DataFrameExec {
+                    df,
+                    projection,
+                    selection,
+                    has_windows,
+                }))
             }
             Sort {
                 input,
@@ -290,7 +308,7 @@ impl DefaultPlanner {
                 apply,
                 schema: _,
                 maintain_order,
-                dynamic_options,
+                options,
             } => {
                 #[cfg(feature = "object")]
                 let input_schema = lp_arena.get(input).schema(lp_arena).clone();
@@ -302,7 +320,7 @@ impl DefaultPlanner {
                 let phys_aggs =
                     self.create_physical_expressions(&aggs, Context::Aggregation, expr_arena)?;
 
-                if let Some(options) = dynamic_options {
+                if let Some(options) = options.dynamic {
                     return Ok(Box::new(GroupByDynamicExec {
                         input,
                         keys: phys_keys,
@@ -311,6 +329,14 @@ impl DefaultPlanner {
                     }));
                 }
 
+                if let Some(options) = options.rolling {
+                    return Ok(Box::new(GroupByRollingExec {
+                        input,
+                        aggs: phys_aggs,
+                        options,
+                    }));
+                }
+
                 // We first check if we can partition the groupby on the latest moment.
                 // TODO: fix this brittle/ buggy state and implement partitioned groupby's in eager
                 let mut partitionable = true;
@@ -335,7 +361,6 @@ impl DefaultPlanner {
                                 AExpr::SortBy { .. }
                                     | AExpr::Filter { .. }
                                     | AExpr::BinaryExpr { .. }
-                                    | AExpr::BinaryFunction { .. }
                                     | AExpr::Function { .. }
                             )
                         };
@@ -442,10 +467,15 @@ impl DefaultPlanner {
                 )))
             }
             HStack { input, exprs, .. } => {
+                let has_windows = exprs.iter().any(|node| has_window_aexpr(*node, expr_arena));
                 let input = self.create_physical_plan(input, lp_arena, expr_arena)?;
                 let phys_expr =
                     self.create_physical_expressions(&exprs, Context::Default, expr_arena)?;
-                Ok(Box::new(StackExec::new(input, phys_expr)))
+                Ok(Box::new(StackExec {
+                    input,
+                    has_windows,
+                    expr: phys_expr,
+                }))
             }
             Udf {
                 input, function, ..
@@ -511,6 +541,7 @@ impl DefaultPlanner {
                     function,
                     phys_function,
                     options,
+                    expr: node_to_exp(expression, expr_arena),
                 }))
             }
             Literal(value) => Ok(Arc::new(LiteralExpr::new(
@@ -616,6 +647,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -637,6 +669,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -658,6 +691,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -679,6 +713,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -700,6 +735,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -723,6 +759,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -744,6 +781,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -765,6 +803,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -786,6 +825,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -807,6 +847,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -832,6 +873,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -858,6 +900,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -891,6 +934,7 @@ impl DefaultPlanner {
                                     function,
                                     expr: node_to_exp(expression, expr_arena),
                                     collect_groups: ApplyOptions::ApplyFlat,
+                                    auto_explode: false,
                                 }))
                             }
                         }
@@ -938,22 +982,7 @@ impl DefaultPlanner {
                     function,
                     expr: node_to_exp(expression, expr_arena),
                     collect_groups: options.collect_groups,
-                }))
-            }
-            BinaryFunction {
-                input_a,
-                input_b,
-                function,
-                output_field,
-            } => {
-                let input_a = self.create_physical_expr(input_a, ctxt, expr_arena)?;
-                let input_b = self.create_physical_expr(input_b, ctxt, expr_arena)?;
-                Ok(Arc::new(BinaryFunctionExpr {
-                    input_a,
-                    input_b,
-                    function,
-                    output_field,
-                    expr: node_to_exp(expression, expr_arena),
+                    auto_explode: options.auto_explode,
                 }))
             }
             Shift { input, periods } => {
@@ -988,6 +1017,7 @@ impl DefaultPlanner {
                     function,
                     expr: node_to_exp(expression, expr_arena),
                     collect_groups: ApplyOptions::ApplyGroups,
+                    auto_explode: false,
                 }))
             }
             Duplicated(expr) => {
@@ -1001,6 +1031,7 @@ impl DefaultPlanner {
                     function,
                     expr: node_to_exp(expression, expr_arena),
                     collect_groups: ApplyOptions::ApplyGroups,
+                    auto_explode: false,
                 }))
             }
             IsUnique(expr) => {
@@ -1014,6 +1045,7 @@ impl DefaultPlanner {
                     function,
                     expr: node_to_exp(expression, expr_arena),
                     collect_groups: ApplyOptions::ApplyGroups,
+                    auto_explode: false,
                 }))
             }
             Explode(expr) => {
@@ -1027,6 +1059,7 @@ impl DefaultPlanner {
                     function,
                     expr: node_to_exp(expression, expr_arena),
                     collect_groups: ApplyOptions::ApplyFlat,
+                    auto_explode: false,
                 }))
             }
             Wildcard => panic!("should be no wildcard at this point"),
