@@ -1,6 +1,15 @@
 use super::*;
+use polars_arrow::prelude::QuantileInterpolOptions;
 use polars_core::series::ops::NullBehavior;
 
+pub(crate) fn load_df() -> DataFrame {
+    df!("a" => &[1, 2, 3, 4, 5],
+                 "b" => &["a", "a", "b", "c", "c"],
+                 "c" => &[1, 2, 3, 4, 5]
+    )
+    .unwrap()
+}
+
 #[test]
 fn test_lazy_ternary() {
     let df = get_df()
@@ -30,7 +39,7 @@ fn test_lazy_with_column() {
     let df = get_df()
         .lazy()
         .with_column(lit(10).alias("foo"))
-        .select(&[col("foo"), col("sepal.width")])
+        .select([col("foo"), col("sepal.width")])
         .collect()
         .unwrap();
     println!("{:?}", df);
@@ -42,7 +51,7 @@ fn test_lazy_exec() {
     let new = df
         .clone()
         .lazy()
-        .select(&[col("sepal.width"), col("variety")])
+        .select([col("sepal.width"), col("variety")])
         .sort("sepal.width", false)
         .collect();
     println!("{:?}", new);
@@ -54,7 +63,7 @@ fn test_lazy_exec() {
         .unwrap();
 
     let check = new.column("sepal.width").unwrap().f64().unwrap().gt(3.4);
-    assert!(check.all_true())
+    assert!(check.all())
 }
 
 #[test]
@@ -62,7 +71,7 @@ fn test_lazy_alias() {
     let df = get_df();
     let new = df
         .lazy()
-        .select(&[col("sepal.width").alias("petals"), col("sepal.width")])
+        .select([col("sepal.width").alias("petals"), col("sepal.width")])
         .collect()
         .unwrap();
     assert_eq!(new.get_column_names(), &["petals", "sepal.width"]);
@@ -107,7 +116,7 @@ fn test_lazy_udf() {
     let df = get_df();
     let new = df
         .lazy()
-        .select(&[col("sepal.width").map(|s| Ok(s * 200.0), GetOutput::same_type())])
+        .select([col("sepal.width").map(|s| Ok(s * 200.0), GetOutput::same_type())])
         .collect()
         .unwrap();
     assert_eq!(
@@ -158,9 +167,9 @@ fn test_lazy_pushdown_through_agg() {
             col("sepal.length").min(),
             col("petal.length").min().alias("foo"),
         ])
-        .select(&[col("foo")])
+        .select([col("foo")])
         // second selection is to test if optimizer can handle that
-        .select(&[col("foo").alias("bar")])
+        .select([col("foo").alias("bar")])
         .collect()
         .unwrap();
 
@@ -209,7 +218,7 @@ fn test_lazy_shift() {
     let df = get_df();
     let new = df
         .lazy()
-        .select(&[col("sepal.width").alias("foo").shift(2)])
+        .select([col("sepal.width").alias("foo").shift(2)])
         .collect()
         .unwrap();
     assert_eq!(new.column("foo").unwrap().f64().unwrap().get(0), None);
@@ -262,20 +271,12 @@ fn test_lazy_binary_ops() {
     let df = df!("a" => &[1, 2, 3, 4, 5, ]).unwrap();
     let new = df
         .lazy()
-        .select(&[col("a").eq(lit(2)).alias("foo")])
+        .select([col("a").eq(lit(2)).alias("foo")])
         .collect()
         .unwrap();
     assert_eq!(new.column("foo").unwrap().sum::<i32>(), Some(1));
 }
 
-fn load_df() -> DataFrame {
-    df!("a" => &[1, 2, 3, 4, 5],
-                 "b" => &["a", "a", "b", "c", "c"],
-                 "c" => &[1, 2, 3, 4, 5]
-    )
-    .unwrap()
-}
-
 #[test]
 fn test_lazy_query_1() {
     // test on aggregation pushdown
@@ -287,7 +288,7 @@ fn test_lazy_query_1() {
         .filter(col("a").lt(lit(2)))
         .groupby([col("b")])
         .agg([col("b").first(), col("c").first()])
-        .select(&[col("b"), col("c_first")])
+        .select([col("b"), col("c_first")])
         .collect()
         .unwrap();
 }
@@ -303,7 +304,7 @@ fn test_lazy_query_2() {
                 .alias("foo"),
         )
         .filter(col("a").lt(lit(2)))
-        .select(&[col("b"), col("a")]);
+        .select([col("b"), col("a")]);
 
     let new = ldf.collect().unwrap();
     assert_eq!(new.shape(), (1, 2));
@@ -339,11 +340,11 @@ fn test_lazy_query_4() {
                 .apply(|s: Series| Ok(&s - &(s.shift(1))), GetOutput::same_type())
                 .alias("diff_cases"),
         ])
-        .explode(vec![col("day"), col("diff_cases")])
+        .explode([col("day"), col("diff_cases")])
         .join(
             base_df,
-            vec![col("uid"), col("day")],
-            vec![col("uid"), col("day")],
+            [col("uid"), col("day")],
+            [col("uid"), col("day")],
             JoinType::Inner,
         )
         .collect()
@@ -389,29 +390,6 @@ fn test_lazy_query_5() {
     assert_eq!(s.len(), 2);
 }
 
-#[test]
-fn test_lazy_query_6() -> Result<()> {
-    let df = df! {
-        "uid" => [0, 0, 0, 1, 1, 1],
-        "day" => [1, 2, 4, 1, 2, 3],
-        "cumcases" => [10, 12, 15, 25, 30, 41]
-    }
-    .unwrap();
-
-    let out = df
-        .lazy()
-        .groupby([col("uid")])
-        // a double aggregation expression.
-        .agg([pearson_corr(col("day"), col("cumcases")).pow(2.0)])
-        .sort("uid", false)
-        .collect()
-        .unwrap();
-    let s = out.column("pearson_corr")?.f64()?;
-    assert!((s.get(0).unwrap() - 0.994360902255639).abs() < 0.000001);
-    assert!((s.get(1).unwrap() - 0.9552238805970149).abs() < 0.000001);
-    Ok(())
-}
-
 #[test]
 #[cfg(feature = "is_in")]
 fn test_lazy_query_8() -> Result<()> {
@@ -466,8 +444,8 @@ fn test_lazy_query_9() -> Result<()> {
         .lazy()
         .join(
             cities.lazy(),
-            vec![col("Sales.City")],
-            vec![col("Cities.City")],
+            [col("Sales.City")],
+            [col("Cities.City")],
             JoinType::Inner,
         )
         .groupby([col("Cities.Country")])
@@ -483,6 +461,80 @@ fn test_lazy_query_9() -> Result<()> {
     Ok(())
 }
 
+#[test]
+#[cfg(feature = "temporal")]
+fn test_lazy_query_10() {
+    use polars_core::export::chrono::Duration as ChronoDuration;
+    let date = NaiveDate::from_ymd(2021, 3, 5);
+    let x: Series = DatetimeChunked::new_from_naive_datetime(
+        "x",
+        &*vec![
+            NaiveDateTime::new(date, NaiveTime::from_hms(12, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(13, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(14, 0, 0)),
+        ],
+        TimeUnit::Nanoseconds,
+    )
+    .into();
+    let y: Series = DatetimeChunked::new_from_naive_datetime(
+        "y",
+        &*vec![
+            NaiveDateTime::new(date, NaiveTime::from_hms(11, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(11, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(11, 0, 0)),
+        ],
+        TimeUnit::Nanoseconds,
+    )
+    .into();
+    let df = DataFrame::new(vec![x, y]).unwrap();
+    let out = df
+        .lazy()
+        .select(&[(col("x") - col("y")).alias("z")])
+        .collect()
+        .unwrap();
+    let z: Series = DurationChunked::new_from_duration(
+        "z",
+        &*vec![
+            ChronoDuration::hours(1),
+            ChronoDuration::hours(2),
+            ChronoDuration::hours(3),
+        ],
+        TimeUnit::Nanoseconds,
+    )
+    .into();
+    assert!(out.column("z").unwrap().series_equal(&z));
+    let x: Series = DatetimeChunked::new_from_naive_datetime(
+        "x",
+        &*vec![
+            NaiveDateTime::new(date, NaiveTime::from_hms(2, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(3, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(4, 0, 0)),
+        ],
+        TimeUnit::Milliseconds,
+    )
+    .into();
+    let y: Series = DatetimeChunked::new_from_naive_datetime(
+        "y",
+        &*vec![
+            NaiveDateTime::new(date, NaiveTime::from_hms(1, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(1, 0, 0)),
+            NaiveDateTime::new(date, NaiveTime::from_hms(1, 0, 0)),
+        ],
+        TimeUnit::Nanoseconds,
+    )
+    .into();
+    let df = DataFrame::new(vec![x, y]).unwrap();
+    let out = df
+        .lazy()
+        .select(&[(col("x") - col("y")).alias("z")])
+        .collect()
+        .unwrap();
+    assert!(out
+        .column("z")
+        .unwrap()
+        .series_equal(&z.cast(&DataType::Duration(TimeUnit::Milliseconds)).unwrap()));
+}
+
 #[test]
 #[cfg(feature = "temporal")]
 fn test_lazy_query_7() {
@@ -568,7 +620,7 @@ fn test_simplify_expr() {
 #[test]
 fn test_lazy_wildcard() {
     let df = load_df();
-    let new = df.clone().lazy().select(&[col("*")]).collect().unwrap();
+    let new = df.clone().lazy().select([col("*")]).collect().unwrap();
     assert_eq!(new.shape(), (5, 3));
 
     let new = df
@@ -592,34 +644,6 @@ fn test_lazy_reverse() {
         .frame_equal_missing(&df.reverse()))
 }
 
-#[test]
-fn test_lazy_filter_and_rename() {
-    let df = load_df();
-    let lf = df
-        .clone()
-        .lazy()
-        .with_column_renamed("a", "x")
-        .filter(col("x").map(
-            |s: Series| Ok(s.gt(3).into_series()),
-            GetOutput::from_type(DataType::Boolean),
-        ))
-        .select(&[col("x")]);
-
-    let correct = df! {
-        "x" => &[4, 5]
-    }
-    .unwrap();
-    assert!(lf.collect().unwrap().frame_equal(&correct));
-
-    // now we check if the column is rename or added when we don't select
-    let lf = df.lazy().with_column_renamed("a", "x").filter(col("x").map(
-        |s: Series| Ok(s.gt(3).into_series()),
-        GetOutput::from_type(DataType::Boolean),
-    ));
-
-    assert_eq!(lf.collect().unwrap().get_column_names(), &["x", "b", "c"]);
-}
-
 #[test]
 fn test_lazy_agg_scan() {
     let lf = scan_foods_csv;
@@ -667,7 +691,7 @@ fn test_lazy_predicate_pushdown_binary_expr() {
     let df = load_df();
     df.lazy()
         .filter(col("a").eq(col("b")))
-        .select(&[col("c")])
+        .select([col("c")])
         .collect()
         .unwrap();
 }
@@ -723,14 +747,13 @@ fn test_lazy_window_functions() {
     // test if partition aggregation is correct
     let out = df
         .lazy()
-        .select(&[col("groups"), sum("values").over([col("groups")])])
+        .select([col("groups"), sum("values").over([col("groups")])])
         .collect()
         .unwrap();
     assert_eq!(
         Vec::from(out.select_at_idx(1).unwrap().i32().unwrap()),
         correct
     );
-    dbg!(out);
 }
 
 #[test]
@@ -740,8 +763,8 @@ fn test_lazy_double_projection() {
     }
     .unwrap();
     df.lazy()
-        .select(&[col("foo").alias("bar")])
-        .select(&[col("bar")])
+        .select([col("foo").alias("bar")])
+        .select([col("bar")])
         .collect()
         .unwrap();
 }
@@ -754,7 +777,7 @@ fn test_type_coercion() {
     }
     .unwrap();
 
-    let lp = df.lazy().select(&[col("foo") * col("bar")]).logical_plan;
+    let lp = df.lazy().select([col("foo") * col("bar")]).logical_plan;
 
     let mut expr_arena = Arena::new();
     let mut lp_arena = Arena::new();
@@ -916,7 +939,7 @@ fn test_lazy_groupby_sort() {
         .agg([col("b").sort(false).first()])
         .collect()
         .unwrap()
-        .sort("a", false)
+        .sort(["a"], false)
         .unwrap();
 
     assert_eq!(
@@ -930,7 +953,7 @@ fn test_lazy_groupby_sort() {
         .agg([col("b").sort(false).last()])
         .collect()
         .unwrap()
-        .sort("a", false)
+        .sort(["a"], false)
         .unwrap();
 
     assert_eq!(
@@ -954,7 +977,7 @@ fn test_lazy_groupby_sort_by() {
         .agg([col("b").sort_by([col("c")], [true]).first()])
         .collect()
         .unwrap()
-        .sort("a", false)
+        .sort(["a"], false)
         .unwrap();
 
     assert_eq!(
@@ -1183,7 +1206,7 @@ fn test_multiple_explode() -> Result<()> {
             col("b").list().alias("b_list"),
             col("c").list().alias("c_list"),
         ])
-        .explode(vec![col("c_list"), col("b_list")])
+        .explode([col("c_list"), col("b_list")])
         .collect()?;
     assert_eq!(out.shape(), (5, 3));
 
@@ -1294,7 +1317,7 @@ fn test_fold_wildcard() -> Result<()> {
     let out = df1
         .clone()
         .lazy()
-        .select([fold_exprs(lit(0), |a, b| Ok(&a + &b), vec![col("*")]).alias("foo")])
+        .select([fold_exprs(lit(0), |a, b| Ok(&a + &b), [col("*")]).alias("foo")])
         .collect()?;
 
     assert_eq!(
@@ -1305,7 +1328,7 @@ fn test_fold_wildcard() -> Result<()> {
     // test if we don't panic due to wildcard
     let _out = df1
         .lazy()
-        .select([all_exprs(vec![col("*").is_not_null()])])
+        .select([all_exprs([col("*").is_not_null()])])
         .collect()?;
     Ok(())
 }
@@ -1540,7 +1563,7 @@ fn test_exploded_window_function() -> Result<()> {
         .select([
             col("fruits"),
             col("B")
-                .shift_and_fill(1, lit(-1.0))
+                .shift_and_fill(1, lit(-1.0f32))
                 .over([col("fruits")])
                 .explode()
                 .alias("shifted"),
@@ -1548,8 +1571,8 @@ fn test_exploded_window_function() -> Result<()> {
         .collect()?;
 
     assert_eq!(
-        Vec::from(out.column("shifted")?.i32()?),
-        &[Some(-1), Some(3), Some(-1), Some(5), Some(4)]
+        Vec::from(out.column("shifted")?.f32()?),
+        &[Some(-1.0), Some(3.0), Some(-1.0), Some(5.0), Some(4.0)]
     );
     Ok(())
 }
@@ -1874,11 +1897,52 @@ fn test_round_after_agg() -> Result<()> {
         .collect()?;
 
     assert!(out.column("foo")?.f32().is_ok());
+
+    let df = df![
+        "groups" => ["pigeon",
+                 "rabbit",
+                 "rabbit",
+                 "Chris",
+                 "pigeon",
+                 "fast",
+                 "fast",
+                 "pigeon",
+                 "rabbit",
+                 "Chris"],
+        "b" => [5409, 4848, 4864, 3540, 8103, 3083, 8575, 9963, 8809, 5425],
+        "c" => [0.4517241160719615,
+                  0.2551467646274673,
+                  0.8682045191407308,
+                  0.9925316385786037,
+                  0.5392027792928116,
+                  0.7633847828107002,
+                  0.7967295231651537,
+                  0.01444779067224733,
+                  0.23807484087472652,
+                  0.10985868798350984]
+    ]?;
+
+    let out = df
+        .lazy()
+        .groupby_stable([col("groups")])
+        .agg([((col("b") * col("c")).sum() / col("b").sum())
+            .round(2)
+            .alias("foo")])
+        .collect()?;
+
+    let out = out.column("foo")?;
+    let out = out.f64()?;
+
+    assert_eq!(
+        Vec::from(out),
+        &[Some(0.3), Some(0.41), Some(0.46), Some(0.79)]
+    );
+
     Ok(())
 }
 
 #[test]
-fn test_power_in_agg_list() -> Result<()> {
+fn test_power_in_agg_list1() -> Result<()> {
     let df = fruits_cars();
 
     // this test if the group tuples are correctly updated after
@@ -1970,7 +2034,7 @@ fn test_exclude_regex() -> Result<()> {
     let df = fruits_cars();
     let out = df
         .lazy()
-        .select([col("*").exclude("^(fruits|cars)$")])
+        .select([col("*").exclude(["^(fruits|cars)$"])])
         .collect()?;
 
     assert_eq!(out.get_column_names(), &["A", "B"]);
@@ -2472,31 +2536,39 @@ fn test_agg_unique_first() -> Result<()> {
 }
 
 #[test]
-fn test_parquet_exec() -> Result<()> {
-    // filter
-    for par in [true, false] {
-        let out = scan_foods_parquet(par)
-            .filter(col("category").eq(lit("seafood")))
-            .collect()?;
-        assert_eq!(out.shape(), (8, 4));
-    }
+#[cfg(feature = "is_in")]
+fn test_is_in() -> Result<()> {
+    let df = fruits_cars();
 
-    // project
-    for par in [true, false] {
-        let out = scan_foods_parquet(par)
-            .select([col("category"), col("sugars_g")])
-            .collect()?;
-        assert_eq!(out.shape(), (27, 2));
-    }
+    // // this will be executed by apply
+    let out = df
+        .clone()
+        .lazy()
+        .groupby_stable([col("fruits")])
+        .agg([col("cars").is_in(col("cars").filter(col("cars").eq(lit("beetle"))))])
+        .collect()?;
+    let out = out.column("cars").unwrap();
+    let out = out.explode()?;
+    let out = out.bool().unwrap();
+    assert_eq!(
+        Vec::from(out),
+        &[Some(true), Some(false), Some(true), Some(true), Some(true)]
+    );
 
-    // project + filter
-    for par in [true, false] {
-        let out = scan_foods_parquet(par)
-            .select([col("category"), col("sugars_g")])
-            .filter(col("category").eq(lit("seafood")))
-            .collect()?;
-        assert_eq!(out.shape(), (8, 2));
-    }
+    // this will be executed by map
+    let out = df
+        .lazy()
+        .groupby_stable([col("fruits")])
+        .agg([col("cars").is_in(lit(Series::new("a", ["beetle", "vw"])))])
+        .collect()?;
+
+    let out = out.column("cars").unwrap();
+    let out = out.explode()?;
+    let out = out.bool().unwrap();
+    assert_eq!(
+        Vec::from(out),
+        &[Some(true), Some(false), Some(true), Some(true), Some(true)]
+    );
 
     Ok(())
 }
