@@ -11,10 +11,9 @@ pub mod export;
 pub mod groupby;
 #[cfg(test)]
 mod test;
-mod unit;
 mod window;
 
 pub use {
     calendar::date_range as date_range_vec, duration::Duration, groupby::ClosedWindow,
-    unit::TimeNanoseconds, window::Window,
+    window::Window,
 };
