@@ -5,8 +5,7 @@ import os
 import shutil
 import subprocess
 import tempfile
-import typing as tp
-from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Type, Union
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Type, Union
 
 try:
     from polars.polars import PyExpr, PyLazyFrame, PyLazyGroupBy
@@ -25,8 +24,8 @@ def wrap_ldf(ldf: "PyLazyFrame") -> "LazyFrame":
 
 
 def _prepare_groupby_inputs(
-    by: Optional[Union[str, tp.List[str], "pli.Expr", tp.List["pli.Expr"]]],
-) -> tp.List["PyExpr"]:
+    by: Optional[Union[str, List[str], "pli.Expr", List["pli.Expr"]]],
+) -> List["PyExpr"]:
     if isinstance(by, list):
         new_by = []
         for e in by:
@@ -65,18 +64,20 @@ class LazyFrame:
         quote_char: Optional[str] = r'"',
         skip_rows: int = 0,
         dtypes: Optional[Dict[str, Type[DataType]]] = None,
-        null_values: Optional[Union[str, tp.List[str], Dict[str, str]]] = None,
+        null_values: Optional[Union[str, List[str], Dict[str, str]]] = None,
         ignore_errors: bool = False,
         cache: bool = True,
-        with_column_names: Optional[Callable[[tp.List[str]], tp.List[str]]] = None,
+        with_column_names: Optional[Callable[[List[str]], List[str]]] = None,
         infer_schema_length: Optional[int] = 100,
         n_rows: Optional[int] = None,
         low_memory: bool = False,
+        rechunk: bool = True,
+        offset_schema_inference: int = 0,
     ) -> "LazyFrame":
         """
         See Also: `pl.scan_csv`
         """
-        dtype_list: Optional[tp.List[Tuple[str, Type[DataType]]]] = None
+        dtype_list: Optional[List[Tuple[str, Type[DataType]]]] = None
         if dtypes is not None:
             dtype_list = []
             for k, v in dtypes.items():
@@ -99,31 +100,40 @@ class LazyFrame:
             processed_null_values,
             infer_schema_length,
             with_column_names,
+            rechunk,
+            offset_schema_inference,
         )
         return self
 
     @staticmethod
     def scan_parquet(
-        file: str, n_rows: Optional[int] = None, cache: bool = True
+        file: str,
+        n_rows: Optional[int] = None,
+        cache: bool = True,
+        parallel: bool = True,
+        rechunk: bool = True,
     ) -> "LazyFrame":
         """
         See Also: `pl.scan_parquet`
         """
 
         self = LazyFrame.__new__(LazyFrame)
-        self._ldf = PyLazyFrame.new_from_parquet(file, n_rows, cache)
+        self._ldf = PyLazyFrame.new_from_parquet(file, n_rows, cache, parallel, rechunk)
         return self
 
     @staticmethod
     def scan_ipc(
-        file: str, n_rows: Optional[int] = None, cache: bool = True
+        file: str,
+        n_rows: Optional[int] = None,
+        cache: bool = True,
+        rechunk: bool = True,
     ) -> "LazyFrame":
         """
         See Also: `pl.scan_ipc`
         """
 
         self = LazyFrame.__new__(LazyFrame)
-        self._ldf = PyLazyFrame.new_from_ipc(file, n_rows, cache)
+        self._ldf = PyLazyFrame.new_from_ipc(file, n_rows, cache, rechunk)
         return self
 
     def pipe(self, func: Callable[..., Any], *args: Any, **kwargs: Any) -> Any:
@@ -141,6 +151,18 @@ class LazyFrame:
         """
         return func(self, *args, **kwargs)
 
+    def _repr_html_(self) -> str:
+        insert = self.describe_plan().replace("\n", "<p></p>")
+
+        return f"""<i>naive plan: (run <b>LazyFrame.describe_optimized_plan()</b> to see the optimized plan)</i>
+<p></p>
+<div>{insert}</div>"""
+
+    def __str__(self) -> str:
+        return f"""naive plan: (run LazyFrame.describe_optimized_plan() to see the optimized plan)
+
+{self.describe_plan()}"""
+
     def describe_plan(self) -> str:
         """
         A string representation of the unoptimized query plan.
@@ -153,6 +175,7 @@ class LazyFrame:
         predicate_pushdown: bool = True,
         projection_pushdown: bool = True,
         simplify_expression: bool = True,
+        slice_pushdown: bool = True,
     ) -> str:
         """
         A string representation of the optimized query plan.
@@ -164,6 +187,7 @@ class LazyFrame:
             projection_pushdown,
             simplify_expression,
             string_cache=False,
+            slice_pushdown=slice_pushdown,
         )
 
         return ldf.describe_optimized_plan()
@@ -246,8 +270,8 @@ class LazyFrame:
 
     def sort(
         self,
-        by: Union[str, "pli.Expr", tp.List[str], tp.List["pli.Expr"]],
-        reverse: Union[bool, tp.List[bool]] = False,
+        by: Union[str, "pli.Expr", List[str], List["pli.Expr"]],
+        reverse: Union[bool, List[bool]] = False,
     ) -> "LazyFrame":
         """
         Sort the DataFrame by:
@@ -268,8 +292,7 @@ class LazyFrame:
         if type(reverse) is bool:
             reverse = [reverse]
 
-        by = pli.expr_to_lit_or_expr(by, str_to_lit=False)
-        by = pli._selection_to_pyexpr_list(by)
+        by = pli.selection_to_pyexpr_list(by)
         return wrap_ldf(self._ldf.sort_by_exprs(by, reverse))
 
     def collect(
@@ -280,6 +303,7 @@ class LazyFrame:
         simplify_expression: bool = True,
         string_cache: bool = False,
         no_optimization: bool = False,
+        slice_pushdown: bool = True,
     ) -> pli.DataFrame:
         """
         Collect into a DataFrame.
@@ -306,6 +330,8 @@ class LazyFrame:
                 global cache when the query is finished.
         no_optimization
             Turn off optimizations.
+        slice_pushdown
+            Slice pushdown optimization.
 
         Returns
         -------
@@ -314,6 +340,7 @@ class LazyFrame:
         if no_optimization:
             predicate_pushdown = False
             projection_pushdown = False
+            slice_pushdown = False
 
         ldf = self._ldf.optimization_toggle(
             type_coercion,
@@ -321,6 +348,7 @@ class LazyFrame:
             projection_pushdown,
             simplify_expression,
             string_cache,
+            slice_pushdown,
         )
         return pli.wrap_df(ldf.collect())
 
@@ -333,6 +361,7 @@ class LazyFrame:
         simplify_expression: bool = True,
         string_cache: bool = True,
         no_optimization: bool = False,
+        slice_pushdown: bool = True,
     ) -> pli.DataFrame:
         """
         Fetch is like a collect operation, but it overwrites the number of rows read by every scan
@@ -359,6 +388,8 @@ class LazyFrame:
             This is needed if you want to join on categorical columns.
         no_optimization
             Turn off optimizations.
+        slice_pushdown
+            Slice pushdown opitmizaiton
 
         Returns
         -------
@@ -367,6 +398,7 @@ class LazyFrame:
         if no_optimization:
             predicate_pushdown = False
             projection_pushdown = False
+            slice_pushdown = False
 
         ldf = self._ldf.optimization_toggle(
             type_coercion,
@@ -374,11 +406,12 @@ class LazyFrame:
             projection_pushdown,
             simplify_expression,
             string_cache,
+            slice_pushdown,
         )
         return pli.wrap_df(ldf.fetch(n_rows))
 
     @property
-    def columns(self) -> tp.List[str]:
+    def columns(self) -> List[str]:
         """
         Get or set column names.
 
@@ -476,12 +509,12 @@ class LazyFrame:
         exprs
             Column or columns to select.
         """
-        exprs = pli._selection_to_pyexpr_list(exprs)
+        exprs = pli.selection_to_pyexpr_list(exprs)
         return wrap_ldf(self._ldf.select(exprs))
 
     def groupby(
         self,
-        by: Union[str, tp.List[str], "pli.Expr", tp.List["pli.Expr"]],
+        by: Union[str, List[str], "pli.Expr", List["pli.Expr"]],
         maintain_order: bool = False,
     ) -> "LazyGroupBy":
         """
@@ -498,44 +531,235 @@ class LazyFrame:
         lgb = self._ldf.groupby(new_by, maintain_order)
         return LazyGroupBy(lgb)
 
+    def groupby_rolling(
+        self,
+        index_column: str,
+        period: str,
+        offset: Optional[str] = None,
+        closed: str = "right",
+    ) -> "LazyGroupBy":
+        """
+        Create rolling groups based on a time column (or index value of type Int32, Int64).
+
+        Different from a rolling groupby the windows are now determined by the individual values and are not of constant
+        intervals. For constant intervals use *groupby_dynamic*
+
+        .. seealso::
+
+            groupby_dynamic
+
+
+        The `period` and `offset` arguments are created with
+        the following string language:
+
+        - 1ns   (1 nanosecond)
+        - 1us   (1 microsecond)
+        - 1ms   (1 millisecond)
+        - 1s    (1 second)
+        - 1m    (1 minute)
+        - 1h    (1 hour)
+        - 1d    (1 day)
+        - 1w    (1 week)
+        - 1mo   (1 calendar month)
+        - 1y    (1 calendar year)
+        - 1i    (1 index count)
+
+        Or combine them:
+        "3d12h4m25s" # 3 days, 12 hours, 4 minutes, and 25 seconds
+
+        In case of a groupby_rolling on an integer column, the windows are defined by:
+
+        - "1i"      # length 1
+        - "10i"     # length 10
+
+        .. warning::
+            This API is experimental and may change without it being considered a breaking change.
+
+        Parameters
+        ----------
+        index_column
+            Column used to group based on the time window.
+            Often to type Date/Datetime
+            This column must be sorted in ascending order. If not the output will not make sense.
+
+            In case of a rolling groupby on indices, dtype needs to be one of {Int32, Int64}. Note that
+            Int32 gets temporarely cast to Int64, so if performance matters use an Int64 column.
+        period
+            length of the window
+        offset
+            offset of the window. Default is -period
+        closed
+            Defines if the window interval is closed or not.
+            Any of {"left", "right", "both" "none"}
+
+        Examples
+        --------
+
+        >>> dates = [
+        ...     "2020-01-01 13:45:48",
+        ...     "2020-01-01 16:42:13",
+        ...     "2020-01-01 16:45:09",
+        ...     "2020-01-02 18:12:48",
+        ...     "2020-01-03 19:45:32",
+        ...     "2020-01-08 23:16:43",
+        ... ]
+        >>> df = pl.DataFrame({"dt": dates, "a": [3, 7, 5, 9, 2, 1]}).with_column(
+        ...     pl.col("dt").str.strptime(pl.Datetime)
+        ... )
+        >>> out = df.groupby_rolling(index_column="dt", period="2d").agg(
+        ...     [
+        ...         pl.sum("a").alias("sum_a"),
+        ...         pl.min("a").alias("min_a"),
+        ...         pl.max("a").alias("max_a"),
+        ...     ]
+        ... )
+        >>> assert out["sum_a"].to_list() == [3, 10, 15, 24, 11, 1]
+        >>> assert out["max_a"].to_list() == [3, 7, 7, 9, 9, 1]
+        >>> assert out["min_a"].to_list() == [3, 3, 3, 3, 2, 1]
+        >>> out
+        shape: (6, 4)
+        ┌─────────────────────┬───────┬───────┬───────┐
+        │ dt                  ┆ a_sum ┆ a_max ┆ a_min │
+        │ ---                 ┆ ---   ┆ ---   ┆ ---   │
+        │ datetime[ms]        ┆ i64   ┆ i64   ┆ i64   │
+        ╞═════════════════════╪═══════╪═══════╪═══════╡
+        │ 2020-01-01 13:45:48 ┆ 3     ┆ 3     ┆ 3     │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┤
+        │ 2020-01-01 16:42:13 ┆ 10    ┆ 7     ┆ 3     │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┤
+        │ 2020-01-01 16:45:09 ┆ 15    ┆ 7     ┆ 3     │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┤
+        │ 2020-01-02 18:12:48 ┆ 24    ┆ 9     ┆ 3     │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┤
+        │ 2020-01-03 19:45:32 ┆ 11    ┆ 9     ┆ 2     │
+        ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌┤
+        │ 2020-01-08 23:16:43 ┆ 1     ┆ 1     ┆ 1     │
+        └─────────────────────┴───────┴───────┴───────┘
+
+
+        """
+
+        if offset is None:
+            offset = f"-{period}"
+
+        lgb = self._ldf.groupby_rolling(
+            index_column,
+            period,
+            offset,
+            closed,
+        )
+        return LazyGroupBy(lgb)
+
     def groupby_dynamic(
         self,
-        time_column: str,
+        index_column: str,
         every: str,
         period: Optional[str] = None,
         offset: Optional[str] = None,
         truncate: bool = True,
         include_boundaries: bool = False,
         closed: str = "right",
-        by: Optional[Union[str, tp.List[str], "pli.Expr", tp.List["pli.Expr"]]] = None,
+        by: Optional[Union[str, List[str], "pli.Expr", List["pli.Expr"]]] = None,
     ) -> "LazyGroupBy":
+        """
+        Groups based on a time value (or index value of type Int32, Int64). Time windows are calculated and rows are assigned to windows.
+        Different from a normal groupby is that a row can be member of multiple groups. The time/index window could
+        be seen as a rolling window, with a window size determined by dates/times/values instead of slots in the DataFrame.
+
+        .. seealso::
+
+            groupby_rolling
+
+        A window is defined by:
+
+        - every: interval of the window
+        - period: length of the window
+        - offset: offset of the window
+
+        The `every`, `period` and `offset` arguments are created with
+        the following string language:
+
+        - 1ns   (1 nanosecond)
+        - 1us   (1 microsecond)
+        - 1ms   (1 millisecond)
+        - 1s    (1 second)
+        - 1m    (1 minute)
+        - 1h    (1 hour)
+        - 1d    (1 day)
+        - 1w    (1 week)
+        - 1mo   (1 calendar month)
+        - 1y    (1 calendar year)
+        - 1i    (1 index count)
+
+        Or combine them:
+        "3d12h4m25s" # 3 days, 12 hours, 4 minutes, and 25 seconds
+
+        In case of a groupby_dynamic on an integer column, the windows are defined by:
+
+        - "1i"      # length 1
+        - "10i"     # length 10
+
+        .. warning::
+            This API is experimental and may change without it being considered a breaking change.
+
+        Parameters
+        ----------
+        index_column
+            Column used to group based on the time window.
+            Often to type Date/Datetime
+            This column must be sorted in ascending order. If not the output will not make sense.
+
+            In case of a dynamic groupby on indices, dtype needs to be one of {Int32, Int64}. Note that
+            Int32 gets temporarely cast to Int64, so if performance matters use an Int64 column.
+        every
+            interval of the window
+        period
+            length of the window, if None it is equal to 'every'
+        offset
+            offset of the window
+        truncate
+            truncate the time value to the window lower bound
+        include_boundaries
+            add the lower and upper bound of the window to the "_lower_bound" and "_upper_bound" columns
+            this will impact performance because it's harder to parallelize
+        closed
+            Defines if the window interval is closed or not.
+            Any of {"left", "right", "both" "none"}
+        by
+            Also group by this column/these columns
+
+        """
+
         if period is None:
             period = every
         if offset is None:
             offset = "0ns"
         by = _prepare_groupby_inputs(by)
         lgb = self._ldf.groupby_dynamic(
-            time_column, every, period, offset, truncate, include_boundaries, closed, by
+            index_column,
+            every,
+            period,
+            offset,
+            truncate,
+            include_boundaries,
+            closed,
+            by,
         )
         return LazyGroupBy(lgb)
 
     def join(
         self,
         ldf: "LazyFrame",
-        left_on: Optional[
-            Union[str, "pli.Expr", tp.List[Union[str, "pli.Expr"]]]
-        ] = None,
-        right_on: Optional[
-            Union[str, "pli.Expr", tp.List[Union[str, "pli.Expr"]]]
-        ] = None,
-        on: Optional[Union[str, "pli.Expr", tp.List[Union[str, "pli.Expr"]]]] = None,
+        left_on: Optional[Union[str, "pli.Expr", List[Union[str, "pli.Expr"]]]] = None,
+        right_on: Optional[Union[str, "pli.Expr", List[Union[str, "pli.Expr"]]]] = None,
+        on: Optional[Union[str, "pli.Expr", List[Union[str, "pli.Expr"]]]] = None,
         how: str = "inner",
         suffix: str = "_right",
         allow_parallel: bool = True,
         force_parallel: bool = False,
-        asof_by: Optional[Union[str, tp.List[str]]] = None,
-        asof_by_left: Optional[Union[str, tp.List[str]]] = None,
-        asof_by_right: Optional[Union[str, tp.List[str]]] = None,
+        asof_by: Optional[Union[str, List[str]]] = None,
+        asof_by_left: Optional[Union[str, List[str]]] = None,
+        asof_by_right: Optional[Union[str, List[str]]] = None,
     ) -> "LazyFrame":
         """
         Add a join operation to the Logical Plan.
@@ -590,13 +814,13 @@ class LazyFrame:
                 )
             )
 
-        left_on_: Optional[tp.List[Union[str, pli.Expr]]]
+        left_on_: Optional[List[Union[str, pli.Expr]]]
         if isinstance(left_on, (str, pli.Expr)):
             left_on_ = [left_on]
         else:
             left_on_ = left_on
 
-        right_on_: Optional[tp.List[Union[str, pli.Expr]]]
+        right_on_: Optional[List[Union[str, pli.Expr]]]
         if isinstance(right_on, (str, pli.Expr)):
             right_on_ = [right_on]
         else:
@@ -625,13 +849,13 @@ class LazyFrame:
 
         # set asof_by
 
-        left_asof_by_: Union[tp.List[str], None]
+        left_asof_by_: Union[List[str], None]
         if isinstance(asof_by_left, str):
             left_asof_by_ = [asof_by_left]
         else:
             left_asof_by_ = asof_by_left
 
-        right_asof_by_: Union[tp.List[str], None]
+        right_asof_by_: Union[List[str], None]
         if isinstance(asof_by_right, (str, pli.Expr)):
             right_asof_by_ = [asof_by_right]
         else:
@@ -663,9 +887,7 @@ class LazyFrame:
             )
         )
 
-    def with_columns(
-        self, exprs: Union[tp.List["pli.Expr"], "pli.Expr"]
-    ) -> "LazyFrame":
+    def with_columns(self, exprs: Union[List["pli.Expr"], "pli.Expr"]) -> "LazyFrame":
         """
         Add or overwrite multiple columns in a DataFrame.
 
@@ -698,7 +920,7 @@ class LazyFrame:
         """
         return self.with_columns([expr])
 
-    def drop(self, columns: Union[str, tp.List[str]]) -> "LazyFrame":
+    def drop(self, columns: Union[str, List[str]]) -> "LazyFrame":
         """
         Remove one or multiple columns from a DataFrame.
 
@@ -715,13 +937,16 @@ class LazyFrame:
 
     def with_column_renamed(self, existing_name: str, new_name: str) -> "LazyFrame":
         """
+        .. deprecated:: 0.12.13
+            Use :func:rename instead
+
         Rename a column in the DataFrame
         """
         return wrap_ldf(self._ldf.with_column_renamed(existing_name, new_name))
 
     def rename(self, mapping: Dict[str, str]) -> "LazyFrame":
         """
-        Rename column names. This does not preserve column order.
+        Rename column names.
 
         Parameters
         ----------
@@ -924,7 +1149,7 @@ class LazyFrame:
         return wrap_ldf(self._ldf.quantile(quantile, interpolation))
 
     def explode(
-        self, columns: Union[str, tp.List[str], "pli.Expr", tp.List["pli.Expr"]]
+        self, columns: Union[str, List[str], "pli.Expr", List["pli.Expr"]]
     ) -> "LazyFrame":
         """
         Explode lists to long format.
@@ -974,8 +1199,6 @@ class LazyFrame:
         ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌┤
         │ ...     ┆ ... │
         ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌┤
-        │ c       ┆ 5   │
-        ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌┤
         │ a       ┆ 6   │
         ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌┤
         │ b       ┆ 2   │
@@ -986,13 +1209,13 @@ class LazyFrame:
         └─────────┴─────┘
 
         """
-        columns = pli._selection_to_pyexpr_list(columns)
+        columns = pli.selection_to_pyexpr_list(columns)
         return wrap_ldf(self._ldf.explode(columns))
 
     def drop_duplicates(
         self,
         maintain_order: bool = False,
-        subset: Optional[Union[tp.List[str], str]] = None,
+        subset: Optional[Union[List[str], str]] = None,
     ) -> "LazyFrame":
         """
         Drop duplicate rows from this DataFrame.
@@ -1002,9 +1225,7 @@ class LazyFrame:
             subset = [subset]
         return wrap_ldf(self._ldf.drop_duplicates(maintain_order, subset))
 
-    def drop_nulls(
-        self, subset: Optional[Union[tp.List[str], str]] = None
-    ) -> "LazyFrame":
+    def drop_nulls(self, subset: Optional[Union[List[str], str]] = None) -> "LazyFrame":
         """
         Drop rows with null values from this DataFrame.
 
@@ -1085,7 +1306,7 @@ class LazyFrame:
         return wrap_ldf(self._ldf.drop_nulls(subset))
 
     def melt(
-        self, id_vars: Union[str, tp.List[str]], value_vars: Union[str, tp.List[str]]
+        self, id_vars: Union[str, List[str]], value_vars: Union[str, List[str]]
     ) -> "LazyFrame":
         """
         Unpivot DataFrame to long format.
@@ -1144,7 +1365,7 @@ class LazyGroupBy:
     def __init__(self, lgb: "PyLazyGroupBy"):
         self.lgb = lgb
 
-    def agg(self, aggs: Union[tp.List["pli.Expr"], "pli.Expr"]) -> "LazyFrame":
+    def agg(self, aggs: Union[List["pli.Expr"], "pli.Expr"]) -> "LazyFrame":
         """
         Describe the aggregation that need to be done on a group.
 
@@ -1168,7 +1389,7 @@ class LazyGroupBy:
         ... )  # doctest: +SKIP
 
         """
-        aggs = pli._selection_to_pyexpr_list(aggs)
+        aggs = pli.selection_to_pyexpr_list(aggs)
         return wrap_ldf(self.lgb.agg(aggs))
 
     def head(self, n: int = 5) -> "LazyFrame":
