@@ -66,8 +66,10 @@ def test_init_ndarray() -> None:
     truth = pl.DataFrame({"column_0": [1, 2], "column_1": [3, 4]})
     assert df.frame_equal(truth)
 
-    df = pl.DataFrame(np.array([[1, 2], [3, 4]]), orient="row")
-    truth = pl.DataFrame({"column_0": [1, 3], "column_1": [2, 4]})
+    df = pl.DataFrame([[1, 2.0, "a"], [None, None, None]], orient="row")
+    truth = pl.DataFrame(
+        {"column_0": [1, None], "column_1": [2.0, None], "column_2": ["a", None]}
+    )
     assert df.frame_equal(truth)
 
     # TODO: Uncomment tests below when removing deprecation warning
@@ -487,6 +489,10 @@ def test_pivot() -> None:
     assert gb.count().shape == (2, 6)
     assert gb.median().shape == (2, 6)
 
+    for agg_fn in ["sum", "min", "max", "mean", "count", "median", "mean"]:
+        out = df.pivot(values="c", index="b", columns="a", aggregate_fn=agg_fn)
+        assert out.shape == (2, 6)
+
 
 def test_join() -> None:
     df_left = pl.DataFrame(
@@ -631,13 +637,15 @@ def test_file_buffer() -> None:
     f = BytesIO()
     f.write(b"1,2,3,4,5,6\n7,8,9,10,11,12")
     f.seek(0)
-    df = pl.DataFrame.read_csv(f, has_header=False)
+    df = pl.read_csv(f, has_header=False)
     assert df.shape == (2, 6)
-    f.seek(0)
 
+    f = BytesIO()
+    f.write(b"1,2,3,4,5,6\n7,8,9,10,11,12")
+    f.seek(0)
     # check if not fails on TryClone and Length impl in file.rs
     with pytest.raises(RuntimeError) as e:
-        df.read_parquet(f)
+        pl.read_parquet(f)
     assert "Invalid Parquet file" in str(e.value)
 
 
@@ -870,13 +878,13 @@ def test_read_csv_categorical() -> None:
     f = BytesIO()
     f.write(b"col1,col2,col3,col4,col5,col6\n'foo',2,3,4,5,6\n'bar',8,9,10,11,12")
     f.seek(0)
-    df = pl.DataFrame.read_csv(f, has_header=True, dtypes={"col1": pl.Categorical})
+    df = pl.read_csv(f, has_header=True, dtypes={"col1": pl.Categorical})
     assert df["col1"].dtype == pl.Categorical
 
 
 def test_df_apply() -> None:
     df = pl.DataFrame({"a": ["foo", "bar", "2"], "b": [1, 2, 3], "c": [1.0, 2.0, 3.0]})
-    out = df.apply(lambda x: len(x), None)
+    out = df.apply(lambda x: len(x), None).to_series()
     assert out.sum() == 9
 
 
@@ -1251,10 +1259,25 @@ def test_slicing() -> None:
 
 def test_apply_list_return() -> None:
     df = pl.DataFrame({"start": [1, 2], "end": [3, 5]})
-    out = df.apply(lambda r: pl.Series(range(r[0], r[1] + 1)))  # type: ignore
+    out = df.apply(lambda r: pl.Series(range(r[0], r[1] + 1))).to_series()
     assert out.to_list() == [[1, 2, 3], [2, 3, 4, 5]]
 
 
+def test_apply_dataframe_return() -> None:
+    df = pl.DataFrame({"a": [1, 2, 3], "b": ["c", "d", None]})
+
+    out = df.apply(lambda row: (row[0] * 10, "foo", True, row[-1]))
+    expected = pl.DataFrame(
+        {
+            "column_0": [10, 20, 30],
+            "column_1": ["foo", "foo", "foo"],
+            "column_2": [True, True, True],
+            "column_3": ["c", "d", None],
+        }
+    )
+    assert out.frame_equal(expected, null_equal=True)
+
+
 def test_groupby_cat_list() -> None:  # noqa: W191,E101
     grouped = (
         pl.DataFrame(
@@ -1541,6 +1564,7 @@ def test_fill_null() -> None:
 def test_fill_nan() -> None:
     df = pl.DataFrame({"a": [1, 2], "b": [3.0, float("nan")]})
     assert df.fill_nan(4).frame_equal(pl.DataFrame({"a": [1, 2], "b": [3, 4]}))
+    assert df["b"].fill_nan(5.0).to_list() == [3.0, 5.0]
 
 
 def test_shift_and_fill() -> None:
@@ -1747,3 +1771,23 @@ def test_to_dict(as_series: bool, inner_dtype: Any) -> None:
     for v in s.values():
         assert isinstance(v, inner_dtype)
         assert len(v) == len(df)
+
+
+def test_df_broadcast() -> None:
+    df = pl.DataFrame({"a": [1, 2, 3]})
+    out = df.with_column(pl.Series([[1, 2]]))
+    assert out.shape == (3, 2)
+
+
+def test_product() -> None:
+    df = pl.DataFrame(
+        {
+            "int": [1, 2, 3],
+            "flt": [-1.0, 12.0, 9.0],
+            "bool_0": [True, False, True],
+            "bool_1": [True, True, True],
+        }
+    )
+    out = df.product()
+    expected = pl.DataFrame({"int": [6], "flt": [-108.0], "bool_0": [0], "bool_1": [1]})
+    assert out.frame_equal(expected)
