@@ -106,3 +106,16 @@ def test_list_concat_rolling_window() -> None:
     )
     assert out.shape == (5, 5)
     assert out["A_rolling"].dtype == pl.List
+
+
+def test_list_append() -> None:
+    df = pl.DataFrame({"a": [[1, 2], [1], [1, 2, 3]]})
+
+    out = df.select([pl.col("a").arr.concat(pl.Series([[1, 2]]))])
+    assert out["a"][0].to_list() == [1, 2, 1, 2]
+
+    out = df.select([pl.col("a").arr.concat([1, 4])])
+    assert out["a"][0].to_list() == [1, 2, 1, 4]
+
+    out_s = df["a"].arr.concat(([4, 1]))
+    assert out_s[0].to_list() == [1, 2, 4, 1]
